@@ -24,20 +24,20 @@
 
 #include "logstream.hxx"
 
+#include <cstring>
 #include <iostream>
 #include <fstream>
 #include <sstream>
 #include <algorithm>
-
-#include <boost/foreach.hpp>
+#include <mutex>
 
 #include <simgear/sg_inlines.h>
 #include <simgear/threads/SGThread.hxx>
 #include <simgear/threads/SGQueue.hxx>
-#include <simgear/threads/SGGuard.hxx>
 
 #include <simgear/io/iostreams/sgstream.hxx>
 #include <simgear/misc/sg_path.hxx>
+#include <simgear/misc/strutils.hxx>
 #include <simgear/timing/timestamp.hxx>
 
 #if defined (SG_WINDOWS)
@@ -62,7 +62,7 @@ LogCallback::LogCallback(sgDebugClass c, sgDebugPriority p) :
 
 bool LogCallback::shouldLog(sgDebugClass c, sgDebugPriority p) const
 {
-    
+
     if ((c & m_class) != 0 && p >= m_priority)
         return true;
     if (c == SG_OSG) // always have OSG logging as it OSG logging is configured separately.
@@ -158,11 +158,18 @@ public:
             << std::setw(10)
             << std::left
             << debugClassToString(c)
-            << " " 
-            << file 
-            << ":" 
-            << line 
-            << ":" 
+            ;
+        if (file) {
+            /* <line> can be -ve to indicate that m_fileLine was false, but we
+            want to show file:line information regardless of m_fileLine. */
+            m_file
+                << file
+                << ":"
+                << abs(line)
+                << ": "
+                ;
+            }
+        m_file
             << message << std::endl;
         //m_file << debugClassToString(c) << ":" << (int)p
         //    << ":" << file << ":" << line << ":" << message << std::endl;
@@ -194,7 +201,13 @@ public:
     {
         if (!shouldLog(c, p)) return;
         //fprintf(stderr, "%s\n", aMessage.c_str());
-        fprintf(stderr, "%8.2f [%.8s]:%-10s %s\n", logTimer.elapsedMSec()/1000.0, debugPriorityToString(p), debugClassToString(c), aMessage.c_str());
+
+        if (file && line > 0) {
+            fprintf(stderr, "%8.2f %s:%i: [%.8s]:%-10s %s\n", logTimer.elapsedMSec()/1000.0, file, line, debugPriorityToString(p), debugClassToString(c), aMessage.c_str());
+        }
+        else {
+            fprintf(stderr, "%8.2f [%.8s]:%-10s %s\n", logTimer.elapsedMSec()/1000.0, debugPriorityToString(p), debugClassToString(c), aMessage.c_str());
+        }
         //    file, line, aMessage.c_str());
         //fprintf(stderr, "%s:%d:%s:%d:%s\n", debugClassToString(c), p,
         //    file, line, aMessage.c_str());
@@ -289,10 +302,10 @@ public:
          *    window; stdout/stderr will not appear (except in logfiles as they do now)
          * 4) When started from the Console (with --console) open a new console window
          * 5) Ensure that IO redirection still works when started from the console
-         * 
+         *
          * Notes:
          * 1) fgfs needs to be a GUI subsystem app - which it already is
-         * 2) What can't be done is to make the cmd prompt run fgfs synchronously; 
+         * 2) What can't be done is to make the cmd prompt run fgfs synchronously;
          * this is only something that can be done via "start /wait fgfs".
          */
 
@@ -316,7 +329,7 @@ public:
 			}
         } else {
             /*
-            * Attempt to attach to the console process of the parent process; when launched from cmd.exe this should be the console, 
+            * Attempt to attach to the console process of the parent process; when launched from cmd.exe this should be the console,
             * when launched via the RUN menu explorer, or another GUI app that wasn't started from the console this will fail.
             * When it fails we will redirect to the NUL device. This is to ensure that we have valid streams.
             * Later on in the initialisation sequence the --console option will be processed and this will cause the requestConsole() to
@@ -346,7 +359,7 @@ public:
             if (!stdout_isNull){
                 if (!m_stdout_isRedirectedAlready)
                     freopen("conout$", "w", stdout);
-                else 
+                else
                     /*
                     * for already redirected streams we need to attach the stream to the OS handle that is open.
                     * - this comes from part of the answer http://stackoverflow.com/a/13841522
@@ -365,7 +378,7 @@ public:
             }
         }
         //http://stackoverflow.com/a/25927081
-        //Clear the error state for each of the C++ standard stream objects. 
+        //Clear the error state for each of the C++ standard stream objects.
         std::wcout.clear();
         std::cout.clear();
         std::wcerr.clear();
@@ -374,9 +387,14 @@ public:
 
         m_callbacks.push_back(new StderrLogCallback(m_logClass, m_logPriority));
         m_consoleCallbacks.push_back(m_callbacks.back());
-#if defined (SG_WINDOWS) && !defined(NDEBUG)
-		m_callbacks.push_back(new WinDebugLogCallback(m_logClass, m_logPriority));
-		m_consoleCallbacks.push_back(m_callbacks.back());
+        
+#if defined (SG_WINDOWS)
+        const char* winDebugEnv = ::getenv("SG_WINDEBUG");
+        const bool b = winDebugEnv ? simgear::strutils::to_bool(std::string{winDebugEnv}) : false;
+        if (b) {
+            m_callbacks.push_back(new WinDebugLogCallback(m_logClass, m_logPriority));
+            m_consoleCallbacks.push_back(m_callbacks.back());
+        }
 #endif
     }
 
@@ -385,7 +403,7 @@ public:
         removeCallbacks();
     }
 
-    SGMutex m_lock;
+    std::mutex m_lock;
     SGBlockingQueue<LogEntry> m_entries;
 
     // log entries posted during startup
@@ -407,13 +425,14 @@ public:
     bool m_stdout_isRedirectedAlready = false;
 #endif
     bool m_developerMode = false;
+    bool m_fileLine = false;
 
     // test suite mode.
     bool m_testMode = false;
 
     void startLog()
     {
-        SGGuard<SGMutex> g(m_lock);
+        std::lock_guard<std::mutex> g(m_lock);
         if (m_isRunning) return;
         m_isRunning = true;
         start();
@@ -425,8 +444,11 @@ public:
             return;
         }
 
-        m_startupLogging = on;
-        m_startupEntries.clear();
+        {
+            std::lock_guard<std::mutex> g(m_lock);
+            m_startupLogging = on;
+            m_startupEntries.clear();
+        }
     }
 
     virtual void run()
@@ -435,16 +457,17 @@ public:
             LogEntry entry(m_entries.pop());
             // special marker entry detected, terminate the thread since we are
             // making a configuration change or quitting the app
-            if ((entry.debugClass == SG_NONE) && !strcmp(entry.file, "done")) {
+            if ((entry.debugClass == SG_NONE) && entry.file && !strcmp(entry.file, "done")) {
                 return;
             }
-
-            if (m_startupLogging) {
-                // save to the startup list for not-yet-added callbacks to
-                // pull down on startup
-                m_startupEntries.push_back(entry);
+            {
+                std::lock_guard<std::mutex> g(m_lock);
+                if (m_startupLogging) {
+                    // save to the startup list for not-yet-added callbacks to
+                    // pull down on startup
+                    m_startupEntries.push_back(entry);
+                }
             }
-
             // submit to each installed callback in turn
             for (simgear::LogCallback* cb : m_callbacks) {
                 (*cb)(entry.debugClass, entry.debugPriority,
@@ -455,14 +478,16 @@ public:
 
     bool stop()
     {
-        SGGuard<SGMutex> g(m_lock);
-        if (!m_isRunning) {
-            return false;
-        }
+        {
+            std::lock_guard<std::mutex> g(m_lock);
+            if (!m_isRunning) {
+                return false;
+            }
 
-        // log a special marker value, which will cause the thread to wakeup,
-        // and then exit
-        log(SG_NONE, SG_ALERT, "done", -1, "");
+            // log a special marker value, which will cause the thread to wakeup,
+            // and then exit
+            log(SG_NONE, SG_ALERT, "done", -1, "");
+        }
         join();
 
         m_isRunning = false;
@@ -506,7 +531,7 @@ public:
         PauseThread pause(this);
         m_logPriority = p;
         m_logClass = c;
-		BOOST_FOREACH(simgear::LogCallback* cb, m_consoleCallbacks) {
+                for (auto cb : m_consoleCallbacks) {
 	        cb->setLogLevels(c, p);
 		}
     }
@@ -516,7 +541,7 @@ public:
         // Testing mode, so always log.
         if (m_testMode) return true;
 
-        // SG_OSG (OSG notify) - will always be displayed regardless of FG log settings as OSG log level is configured 
+        // SG_OSG (OSG notify) - will always be displayed regardless of FG log settings as OSG log level is configured
         // separately and thus it makes more sense to allow these message through.
         if (static_cast<unsigned>(p) == static_cast<unsigned>(SG_OSG)) return true;
 
@@ -529,6 +554,10 @@ public:
             const char* fileName, int line, const std::string& msg)
     {
         p = translatePriority(p);
+        if (!m_fileLine) {
+            /* This prevents output of file:line in StderrLogCallback. */
+            line = -line;
+        }
         LogEntry entry(c, p, fileName, line, msg);
         m_entries.push(entry);
     }
@@ -540,7 +569,7 @@ public:
         }
 
         if (in == SG_DEV_ALERT) {
-            return m_developerMode ? SG_POPUP : SG_WARN;
+            return m_developerMode ? SG_ALERT : SG_WARN;
         }
 
         return in;
@@ -550,7 +579,7 @@ public:
 /////////////////////////////////////////////////////////////////////////////
 
 static std::unique_ptr<logstream> global_logstream;
-static SGMutex global_logStreamLock;
+static std::mutex global_logStreamLock;
 
 logstream::logstream()
 {
@@ -575,6 +604,10 @@ void logstream::setDeveloperMode(bool devMode)
     d->m_developerMode = devMode;
 }
 
+void logstream::setFileLine(bool fileLine)
+{
+    d->m_fileLine = fileLine;
+}
 
 void
 logstream::addCallback(simgear::LogCallback* cb)
@@ -714,7 +747,7 @@ sglog()
     // http://www.aristeia.com/Papers/DDJ_Jul_Aug_2004_revised.pdf
     // in the absence of portable memory barrier ops in Simgear,
     // let's keep this correct & safe
-    SGGuard<SGMutex> g(global_logStreamLock);
+    std::lock_guard<std::mutex> g(global_logStreamLock);
 
     if( !global_logstream )
         global_logstream.reset(new logstream);
@@ -788,14 +821,14 @@ namespace simgear
 {
 
 void requestConsole()
-{ 
+{
     sglog().requestConsole();
 }
 
 
 void shutdownLogging()
 {
-    SGGuard<SGMutex> g(global_logStreamLock);
+    std::lock_guard<std::mutex> g(global_logStreamLock);
     global_logstream.reset();
 }
 
