@@ -1,3 +1,4 @@
+#include <cassert>
 #include <cstdlib>
 #include <iostream>
 #include <map>
@@ -5,8 +6,6 @@
 #include <errno.h>
 #include <fcntl.h>
 
-#include <boost/algorithm/string/case_conv.hpp>
-
 #include <simgear/simgear_config.h>
 
 #include "test_HTTP.hxx"
