@@ -587,6 +587,40 @@ namespace simgear {
         // reached end - longer wins
         return v1parts.size() - v2parts.size();
     }
+    
+    bool compareVersionToWildcard(const std::string& aVersion, const std::string& aCandidate)
+    {
+        if (aCandidate == aVersion) {
+            return true;
+        }
+
+        // examine each dot-seperated component in turn, supporting a wildcard
+        // in the versions from the catalog.
+        string_list parts = split(aVersion, ".");
+        string_list candidateParts = split(aCandidate, ".");
+
+        const size_t partCount = parts.size();
+        const size_t candidatePartCount =  candidateParts.size();
+        
+        bool previousCandidatePartWasWildcard = false;
+
+        for (unsigned int p=0; p < partCount; ++p) {
+            // candidate string is too short, can match if it ended with wildcard
+            // part. This allows candidate '2016.*' to match '2016.1.2' and so on
+            if (candidatePartCount <= p) {
+                return previousCandidatePartWasWildcard;
+            }
+
+            if (candidateParts.at(p) == "*") {
+                // always passes
+                previousCandidatePartWasWildcard = true;
+            } else if (parts.at(p) != candidateParts.at(p)) {
+                return false;
+            }
+        }
+
+        return true;
+    }
 
     string join(const string_list& l, const string& joinWith)
     {
@@ -672,8 +706,7 @@ static std::string convertWStringToMultiByte(DWORD encoding, const std::wstring&
 std::wstring convertUtf8ToWString(const std::string& a)
 {
 #if defined(SG_WINDOWS)
-    std::wstring_convert<std::codecvt_utf8<wchar_t>, wchar_t> ucs2conv;
-    return ucs2conv.from_bytes(a);
+    return convertMultiByteToWString(CP_UTF8, a);
 #else
     assert(sizeof(wchar_t) == 4);
     std::wstring result;
@@ -723,8 +756,7 @@ std::wstring convertUtf8ToWString(const std::string& a)
 std::string convertWStringToUtf8(const std::wstring& w)
 {
 #if defined(SG_WINDOWS)
-    std::wstring_convert<std::codecvt_utf8<wchar_t>, wchar_t> ucs2conv;
-    return ucs2conv.to_bytes(w);
+    return convertWStringToMultiByte(CP_UTF8, w);
 #else
     assert(sizeof(wchar_t) == 4);
     std::string result;
@@ -899,6 +931,69 @@ std::string encodeHex(const unsigned char* rawBytes, unsigned int length)
   return hex;
 }
 
+std::vector<uint8_t> decodeHex(const std::string& input)
+{
+    std::vector<uint8_t> result;
+    char* ptr = const_cast<char*>(input.data());
+    const char* end = ptr + input.length();
+    
+    bool highNibble = true;
+    uint8_t b = 0;
+    
+    while (ptr != end) {
+        const char c = *ptr;
+        char val = 0;
+        
+        if (c == '0') {
+            val = 0;
+            if ((ptr + 1) < end) {
+                const auto peek = *(ptr + 1);
+                if (peek == 'x') {
+                    // tolerate 0x prefixing
+                    highNibble = true;
+                    ptr += 2; // skip both bytes
+                    continue;
+                }
+            }
+        } else if (isdigit(c)) {
+            val =  c - '0';
+        } else if ((c >= 'A') && (c <= 'F')) {
+            val = c - 'A' + 10;
+        } else if ((c >= 'a') && (c <= 'f')) {
+            val = c - 'a' + 10;
+        } else {
+            // any other input: newline, space, tab, comma...
+            if (!highNibble) {
+                // allow a single digit to work, if we have spacing
+                highNibble = true;
+                result.push_back(b >> 4);
+            }
+            
+            ++ptr;
+            continue;
+        }
+        
+        if (highNibble) {
+            highNibble = false;
+            b = val << 4;
+        } else {
+            highNibble = true;
+            b |= val;
+            result.push_back(b);
+        }
+        
+        ++ptr;
+    }
+    
+    // watch for trailing single digit
+    // this is reqquired so a stirng ending in 0x3 is decoded.
+    if (!highNibble) {
+        result.push_back(b >> 4);
+    }
+    
+    return result;
+}
+    
 // Write an octal backslash-escaped respresentation of 'val' to 'buf'.
 //
 // At least 4 write positions must be available at 'buf'. The result is *not*
@@ -1007,6 +1102,16 @@ std::string unescape(const char* s)
   }
   return r;
 }
+std::string replace(std::string source, const std::string search, const std::string replacement, std::size_t start_pos)
+{
+    if (start_pos < source.length()) {
+        while ((start_pos = source.find(search, start_pos)) != std::string::npos) {
+            source.replace(start_pos, search.length(), replacement);
+            start_pos += replacement.length();
+        }
+    }
+    return source;
+}
 
 string sanitizePrintfFormat(const string& input)
 {
@@ -1033,7 +1138,7 @@ std::string error_string(int errnum)
   retcode = strerror_s(buf, sizeof(buf), errnum);
 #elif defined(_GNU_SOURCE)
   return std::string(strerror_r(errnum, buf, sizeof(buf)));
-#elif (_POSIX_C_SOURCE >= 200112L) || defined(SG_MAC) || defined(__FreeBSD__)
+#elif (_POSIX_C_SOURCE >= 200112L) || defined(SG_MAC) || defined(__FreeBSD__) || defined(__OpenBSD__)
   int retcode;
   // POSIX.1-2001 and POSIX.1-2008
   retcode = strerror_r(errnum, buf, sizeof(buf));
