@@ -157,7 +157,7 @@ static void initContext(naContext c)
     c->error[0] = 0;
     c->userData = 0;
 }
-
+#define BASE_SIZE 256000
 static void initGlobals()
 {
     int i;
@@ -168,10 +168,10 @@ static void initGlobals()
     globals->sem = naNewSem();
     globals->lock = naNewLock();
 
-    globals->allocCount = 256; // reasonable starting value
+    globals->allocCount = BASE_SIZE; // reasonable starting value
     for(i=0; i<NUM_NASAL_TYPES; i++)
         naGC_init(&(globals->pools[i]), i);
-    globals->deadsz = 256;
+    globals->deadsz = BASE_SIZE;
     globals->ndead = 0;
     globals->deadBlocks = naAlloc(sizeof(void*) * globals->deadsz);
 
@@ -235,6 +235,21 @@ void naFreeContext(naContext c)
     if(c->callChild) naFreeContext(c->callChild);
     if(c->callParent) c->callParent->callChild = 0;
     LOCK();
+
+    // 2019-09-21
+    // James adding this to ensure stray stuff in freed contexts gets GCed
+    // this shows up when doing a reset / shutdown of all Nasal - we drop
+    // all our contexts and saved refs, and run a GC pass. We expect *everything*
+    // to be freed but actually the freed contexts often have a ref in their
+    // opStack.
+    //
+    // The underlying cause is likely some operation which leaves a value on
+    // the opstack accidently, but tracing that down requires more Nasal-fu
+    // than I have right now. So instead I'm clearing the stack tops here, so
+    // a freed context looks the same as a new one returned by initContext.
+
+    c->fTop = c->opTop = c->markTop = 0;
+
     c->nextFree = globals->freeContexts;
     globals->freeContexts = c;
     UNLOCK();
@@ -488,7 +503,7 @@ static void setMember(naContext ctx, naRef obj, naRef fld, naRef value)
         return;
     }
     
-    if(!IS_HASH(obj)) ERR(ctx, "non-objects have no members");
+    if(!IS_HASH(obj)) naRuntimeError(ctx, "non-object does not have member: %s", naStr_data(fld));
     naHash_set(obj, fld, value);
     ctx->opTop -= 2;
 }
@@ -833,9 +848,13 @@ naRef naGetSourceFile(naContext ctx, int frame)
 {
     naRef f;
     frame = findFrame(ctx, &ctx, frame);
-    f = ctx->fStack[frame].func;
-    f = PTR(f).func->code;
-    return PTR(f).code->srcFile;
+    if (frame >= 0) {
+        f = ctx->fStack[frame].func;
+        f = PTR(f).func->code;
+        if (!IS_NIL(f) && PTR(f).code)
+            return PTR(f).code->srcFile;
+    }
+    return naNil();
 }
 
 char* naGetError(naContext ctx)
