@@ -18,10 +18,10 @@
 #include <simgear_config.h>
 #include <simgear/package/Catalog.hxx>
 
-#include <boost/foreach.hpp>
 #include <algorithm>
-#include <fstream>
+#include <cassert>
 #include <cstring>
+#include <fstream>
 
 #include <simgear/debug/logstream.hxx>
 #include <simgear/props/props_io.hxx>
@@ -39,59 +39,27 @@ namespace simgear {
 
 namespace pkg {
 
-bool checkVersionString(const std::string& aVersion, const std::string& aCandidate)
-{
-    if (aCandidate == aVersion) {
-        return true;
-    }
-
-    // examine each dot-seperated component in turn, supporting a wildcard
-    // in the versions from the catalog.
-    string_list appVersionParts = simgear::strutils::split(aVersion, ".");
-    string_list catVersionParts = simgear::strutils::split(aCandidate, ".");
-
-    size_t partCount = appVersionParts.size();
-    bool previousCandidatePartWasWildcard = false;
-
-    for (unsigned int p=0; p < partCount; ++p) {
-        // candidate string is too short, can match if it ended with wildcard
-        // part. This allows candidate '2016.*' to match '2016.1.2' and so on
-        if (catVersionParts.size() <= p) {
-            return previousCandidatePartWasWildcard;
-        }
-
-        if (catVersionParts[p] == "*") {
-            // always passes
-            previousCandidatePartWasWildcard = true;
-        } else if (appVersionParts[p] != catVersionParts[p]) {
-            return false;
-        }
-    }
-
-    return true;
-}
-
 bool checkVersion(const std::string& aVersion, SGPropertyNode_ptr props)
 {
-    BOOST_FOREACH(SGPropertyNode* v, props->getChildren("version")) {
+    for (SGPropertyNode* v : props->getChildren("version")) {
         std::string s(v->getStringValue());
-        if (checkVersionString(aVersion, s)) {
+        if (strutils::compareVersionToWildcard(aVersion, s)) {
             return true;
         }
     }
     return false;
 }
-    
+
 SGPropertyNode_ptr alternateForVersion(const std::string& aVersion, SGPropertyNode_ptr props)
 {
     for (auto v : props->getChildren("alternate-version")) {
         for (auto versionSpec : v->getChildren("version")) {
-            if (checkVersionString(aVersion, versionSpec->getStringValue())) {
+            if (strutils::compareVersionToWildcard(aVersion, versionSpec->getStringValue())) {
                 return v;
             }
         }
     }
-    
+
     return {};
 }
 
@@ -152,18 +120,18 @@ protected:
             // check for a version redirect entry
             auto alt = alternateForVersion(ver, props);
             if (alt) {
-                SG_LOG(SG_GENERAL, SG_WARN, "have alternate version of package at:"
+                SG_LOG(SG_GENERAL, SG_INFO, "have alternate version of package at:"
                        << alt->getStringValue("url"));
                 m_owner->processAlternate(alt);
             } else {
-                SG_LOG(SG_GENERAL, SG_WARN, "downloaded catalog " << m_owner->url()
-                       << ", but app version " << ver << " is not comaptible");
+                SG_LOG(SG_GENERAL, SG_DEBUG, "downloaded catalog " << m_owner->url()
+                       << ", but app version " << ver << " is not compatible");
                 m_owner->refreshComplete(Delegate::FAIL_VERSION);
             }
 
             return;
         } // of version check failed
-        
+
         // validate what we downloaded, in case it's now corrupted
         // (i.e someone uploaded bad XML data)
         if (!m_owner->validatePackages()) {
@@ -182,7 +150,7 @@ protected:
         m_owner->writeTimestamp();
         m_owner->refreshComplete(Delegate::STATUS_REFRESHED);
     }
-    
+
     void onFail() override
     {
         // network level failure
@@ -198,9 +166,7 @@ private:
 //////////////////////////////////////////////////////////////////////////////
 
 Catalog::Catalog(Root *aRoot) :
-    m_root(aRoot),
-    m_status(Delegate::FAIL_UNKNOWN),
-    m_retrievedTime(0)
+    m_root(aRoot)
 {
 }
 
@@ -221,7 +187,7 @@ CatalogRef Catalog::createFromPath(Root* aRoot, const SGPath& aPath)
     SGPath xml = aPath;
     xml.append("catalog.xml");
     if (!xml.exists()) {
-        return NULL;
+        return nullptr;
     }
 
     SGPropertyNode_ptr props;
@@ -229,7 +195,7 @@ CatalogRef Catalog::createFromPath(Root* aRoot, const SGPath& aPath)
         props = new SGPropertyNode;
         readProperties(xml, props);
     } catch (sg_exception& ) {
-        return NULL;
+        return nullptr;
     }
 
     bool versionCheckOk = checkVersion(aRoot->applicationVersion(), props);
@@ -241,23 +207,33 @@ CatalogRef Catalog::createFromPath(Root* aRoot, const SGPath& aPath)
         SG_LOG(SG_GENERAL, SG_DEBUG, "creating catalog from:" << aPath);
     }
 
+    // check for the marker file we write, to mark a catalog as disabled
+    const SGPath disableMarkerFile = aPath / "_disabled_";
+
     CatalogRef c = new Catalog(aRoot);
     c->m_installRoot = aPath;
+
+    if (disableMarkerFile.exists()) {
+        c->m_userEnabled = false;
+    }
+
     c->parseProps(props);
     c->parseTimestamp();
 
     if (!c->validatePackages()) {
         c->changeStatus(Delegate::FAIL_VALIDATION);
-    } else if (versionCheckOk) {
+    } else if (!versionCheckOk) {
+         c->changeStatus(Delegate::FAIL_VERSION);
+    } else if (!c->m_userEnabled) {
+          c->changeStatus(Delegate::USER_DISABLED);
+    } else {
         // parsed XML ok, mark status as valid
         c->changeStatus(Delegate::STATUS_SUCCESS);
-    } else {
-        c->changeStatus(Delegate::FAIL_VERSION);
     }
 
     return c;
 }
-    
+
 bool Catalog::validatePackages() const
 {
     for (auto pack : packages()) {
@@ -266,7 +242,7 @@ bool Catalog::validatePackages() const
             return false;
         }
     }
-    
+
     return true;
 }
 
@@ -309,10 +285,10 @@ bool Catalog::removeDirectory()
     Dir d(m_installRoot);
     if (!m_installRoot.exists())
         return true;
-    
+
     return d.remove(true /* recursive */);
 }
-    
+
 PackageList const&
 Catalog::packages() const
 {
@@ -323,7 +299,7 @@ PackageList
 Catalog::packagesMatching(const SGPropertyNode* aFilter) const
 {
     PackageList r;
-    BOOST_FOREACH(PackageRef p, m_packages) {
+    for (auto p : m_packages) {
         if (p->matches(aFilter)) {
             r.push_back(p);
         }
@@ -335,7 +311,7 @@ PackageList
 Catalog::packagesNeedingUpdate() const
 {
     PackageList r;
-    BOOST_FOREACH(PackageRef p, m_packages) {
+    for (auto p : m_packages) {
         if (!p->isInstalled()) {
             continue;
         }
@@ -351,7 +327,7 @@ PackageList
 Catalog::installedPackages() const
 {
   PackageList r;
-  BOOST_FOREACH(PackageRef p, m_packages) {
+  for (auto p : m_packages) {
     if (p->isInstalled()) {
       r.push_back(p);
     }
@@ -592,6 +568,9 @@ Delegate::StatusCode Catalog::status() const
 
 bool Catalog::isEnabled() const
 {
+    if (!m_userEnabled)
+        return false;
+
     switch (m_status) {
     case Delegate::STATUS_SUCCESS:
     case Delegate::STATUS_REFRESHED:
@@ -603,7 +582,37 @@ bool Catalog::isEnabled() const
         return false;
     }
 }
-    
+
+bool Catalog::isUserEnabled() const
+{
+    return m_userEnabled;
+}
+
+void Catalog::setUserEnabled(bool b)
+{
+    if (m_userEnabled == b)
+        return;
+
+    m_userEnabled = b;
+    SGPath disableMarkerFile = installRoot() / "_disabled_";
+    if (m_userEnabled) {
+        sg_ofstream of(disableMarkerFile);
+        of << "1\n"; // touch the file
+    } else {
+        bool ok = disableMarkerFile.remove();
+        if (!ok) {
+            SG_LOG(SG_GENERAL, SG_ALERT, "Failed to remove catalog-disable marker file:" << disableMarkerFile);
+        }
+    }
+
+    Delegate::StatusCode effectiveStatus = m_status;
+    if ((m_status == Delegate::STATUS_SUCCESS) && !m_userEnabled) {
+        effectiveStatus = Delegate::USER_DISABLED;
+    }
+
+    m_root->catalogRefreshStatus(this, effectiveStatus);
+}
+
 void Catalog::processAlternate(SGPropertyNode_ptr alt)
 {
     std::string altId;
@@ -611,19 +620,19 @@ void Catalog::processAlternate(SGPropertyNode_ptr alt)
     if (idPtr) {
         altId = std::string(idPtr);
     }
-    
+
     std::string altUrl;
     if (alt->getStringValue("url")) {
         altUrl = std::string(alt->getStringValue("url"));
     }
-    
+
     CatalogRef existing;
     if (!altId.empty()) {
         existing = root()->getCatalogById(altId);
     } else {
         existing = root()->getCatalogByUrl(altUrl);
     }
-    
+
     if (existing && (existing != this)) {
         // we already have the alternate, so just go quiet here
         changeStatus(Delegate::FAIL_VERSION);
@@ -641,13 +650,13 @@ void Catalog::processAlternate(SGPropertyNode_ptr alt)
         changeStatus(Delegate::FAIL_VERSION);
         return;
     }
-    
+
     SG_LOG(SG_GENERAL, SG_INFO, "Migrating catalog " << id() << " to new URL:" << altUrl);
     setUrl(altUrl);
     Downloader* dl = new Downloader(this, altUrl);
     root()->makeHTTPRequest(dl);
 }
-    
+
 } // of namespace pkg
 
 } // of namespace simgear
