@@ -15,9 +15,7 @@
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
 
-#ifdef HAVE_CONFIG_H
-#  include <simgear_config.h>
-#endif
+#include <simgear_config.h>
 
 #include <simgear/misc/test_macros.hxx>
 
@@ -62,6 +60,9 @@ std::string readFileIntoString(const SGPath& path)
 SGPath global_serverFilesRoot;
 unsigned int global_catalogVersion = 0;
 bool global_failRequests = false;
+bool global_fail747Request = true;
+
+int global_737RequestsToFailCount = 0;
 
 class TestPackageChannel : public TestServerChannel
 {
@@ -98,6 +99,24 @@ public:
             path = "/catalogTest1/movies-data.zip";
         }
 
+        if (path == "/catalogTest1/b747.tar.gz") {
+            if (global_fail747Request) {
+                sendErrorResponse(403, false, "Bad URL");
+                return;
+            } else {
+                path = "/catalogTest1/b747.tar.gz"; // valid path
+            }
+        }
+        
+        if ((path.find("/mirror") == 0) && (path.find("/b737.tar.gz") == 8)) {
+            if (global_737RequestsToFailCount > 0) {
+                sendErrorResponse(404, false, "Mirror failure");
+                --global_737RequestsToFailCount;
+                return;
+            }
+            path = "/catalogTest1/b737.tar.gz";
+        }
+        
         localPath.append(path);
 
       //  SG_LOG(SG_IO, SG_INFO, "local path is:" << localPath.str());
@@ -158,7 +177,7 @@ int parseTest()
     SG_CHECK_EQUAL(cat->description(), "First test catalog");
 
 // check the packages too
-    SG_CHECK_EQUAL(cat->packages().size(), 5);
+    SG_CHECK_EQUAL(cat->packages().size(), 6);
 
     pkg::PackageRef p1 = cat->packages().front();
     SG_CHECK_EQUAL(p1->catalog(), cat.ptr());
@@ -321,7 +340,9 @@ int parseTest()
         SG_VERIFY(p3->matches(queryText.ptr()));
     }
     
-    
+    string_list urls = p3->downloadUrls();
+    SG_CHECK_EQUAL(urls.size(), 3);
+    SG_CHECK_EQUAL(urls.at(1), "http://localhost:2000/mirrorB/b737.tar.gz");
 
     delete root;
     return EXIT_SUCCESS;
@@ -349,7 +370,7 @@ void testAddCatalog(HTTP::Client* cl)
     p.append("org.flightgear.test.catalog1");
     p.append("catalog.xml");
     SG_VERIFY(p.exists());
-    SG_CHECK_EQUAL(root->allPackages().size(), 5);
+    SG_CHECK_EQUAL(root->allPackages().size(), 6);
     SG_CHECK_EQUAL(root->catalogs().size(), 1);
 
     pkg::PackageRef p1 = root->getPackageById("alpha");
@@ -383,6 +404,7 @@ void testInstallPackage(HTTP::Client* cl)
     waitForUpdateComplete(cl, root);
     SG_VERIFY(p1->isInstalled());
     SG_VERIFY(p1->existingInstall() == ins);
+    SG_CHECK_EQUAL(ins->status(), pkg::Delegate::STATUS_SUCCESS);
 
     pkg::PackageRef commonDeps = root->getPackageById("common-sounds");
     SG_VERIFY(commonDeps->existingInstall());
@@ -428,6 +450,7 @@ void testUninstall(HTTP::Client* cl)
 
     ins->uninstall();
 
+    SG_CHECK_EQUAL(ins->status(), pkg::Delegate::STATUS_SUCCESS);
     SG_VERIFY(!ins->path().exists());
 }
 
@@ -1075,12 +1098,98 @@ void updateInvalidToInvalid(HTTP::Client* cl)
     
 }
 
+void testInstallBadPackage(HTTP::Client* cl)
+{
+    global_catalogVersion = 0;
+    global_fail747Request = true;
+    
+    SGPath rootPath(simgear::Dir::current().path());
+    rootPath.append("pkg_install_bad_pkg");
+    simgear::Dir pd(rootPath);
+    pd.removeChildren();
+
+    pkg::RootRef root(new pkg::Root(rootPath, "8.1.2"));
+    // specify a test dir
+    root->setHTTPClient(cl);
+
+    pkg::CatalogRef c = pkg::Catalog::createFromUrl(root.ptr(), "http://localhost:2000/catalogTest1/catalog.xml");
+    waitForUpdateComplete(cl, root);
+
+    pkg::PackageRef p1 = root->getPackageById("org.flightgear.test.catalog1.b747-400");
+    pkg::InstallRef ins = p1->install();
+    
+    bool didFail = false;
+    ins->fail([&didFail, &ins](pkg::Install* ourInstall) {
+        SG_CHECK_EQUAL(ins, ourInstall);
+        didFail = true;
+    });
+    
+    SG_VERIFY(ins->isQueued());
+
+    waitForUpdateComplete(cl, root);
+    SG_VERIFY(!p1->isInstalled());
+    SG_VERIFY(didFail);
+    SG_VERIFY(p1->existingInstall() == ins);
+    SG_CHECK_EQUAL(ins->status(), pkg::Delegate::FAIL_DOWNLOAD);
+    SG_CHECK_EQUAL(ins->path(), rootPath / "org.flightgear.test.catalog1" / "Aircraft" / "b744");
+    
+    // now retry, it should work
+    global_fail747Request = false;
+    
+    auto ins2 = p1->install();
+    SG_CHECK_EQUAL(ins2, p1->existingInstall());
+    root->scheduleToUpdate(ins2);
+    SG_VERIFY(ins2->isQueued());
+
+    didFail = false;
+    waitForUpdateComplete(cl, root);
+    SG_VERIFY(p1->isInstalled());
+    SG_VERIFY(!didFail);
+    SG_CHECK_EQUAL(ins->status(), pkg::Delegate::STATUS_SUCCESS);
+    SG_CHECK_EQUAL(ins->path(), rootPath / "org.flightgear.test.catalog1" / "Aircraft" / "b744");
+}
+
+void testMirrorsFailure(HTTP::Client* cl)
+{
+    global_catalogVersion = 0;
+    // there's three mirrors defined, so se tthe first twom attempts to fail
+    global_737RequestsToFailCount = 2;
+    
+    SGPath rootPath(simgear::Dir::current().path());
+    rootPath.append("pkg_install_mirror_fail");
+    simgear::Dir pd(rootPath);
+    pd.removeChildren();
+
+    pkg::RootRef root(new pkg::Root(rootPath, "8.1.2"));
+    // specify a test dir
+    root->setHTTPClient(cl);
+
+    pkg::CatalogRef c = pkg::Catalog::createFromUrl(root.ptr(), "http://localhost:2000/catalogTest1/catalog.xml");
+    waitForUpdateComplete(cl, root);
+
+    pkg::PackageRef p1 = root->getPackageById("org.flightgear.test.catalog1.b737-NG");
+    pkg::InstallRef ins = p1->install();
+    
+    bool didFail = false;
+    ins->fail([&didFail, &ins](pkg::Install* ourInstall) {
+        SG_CHECK_EQUAL(ins, ourInstall);
+        didFail = true;
+    });
+    
+    waitForUpdateComplete(cl, root);
+    SG_VERIFY(p1->isInstalled());
+    SG_VERIFY(!didFail);
+    SG_VERIFY(p1->existingInstall() == ins);
+    SG_CHECK_EQUAL(ins->status(), pkg::Delegate::STATUS_SUCCESS);
+    
+}
+
 int main(int argc, char* argv[])
 {
-    sglog().setLogLevels( SG_ALL, SG_DEBUG );
+    sglog().setLogLevels( SG_ALL, SG_WARN );
 
     HTTP::Client cl;
-    cl.setMaxConnections(1);
+    cl.setMaxConnections(1); 
 
     global_serverFilesRoot = SGPath(SRC_DIR);
 
@@ -1089,7 +1198,7 @@ int main(int argc, char* argv[])
     parseTest();
 
     parseInvalidTest();
-    
+
     testInstallPackage(&cl);
 
     testUninstall(&cl);
@@ -1115,7 +1224,11 @@ int main(int argc, char* argv[])
     removeInvalidCatalog(&cl);
     
     testVersionMigrateToId(&cl);
+
+    testInstallBadPackage(&cl);
     
-    SG_LOG(SG_GENERAL, SG_INFO, "Successfully passed all tests!");
+    testMirrorsFailure(&cl);
+    
+    cerr << "Successfully passed all tests!" << endl;
     return EXIT_SUCCESS;
 }
