@@ -28,10 +28,9 @@ using std::cerr;
 #else
 # include <boost/algorithm/string/find_iterator.hpp>
 # include <boost/algorithm/string/predicate.hpp>
-# include <boost/algorithm/string/classification.hpp>
-# include <boost/bind.hpp>
 # include <boost/functional/hash.hpp>
 # include <boost/range.hpp>
+
 # include <simgear/compiler.h>
 # include <simgear/debug/logstream.hxx>
 # include <simgear/sg_inlines.h>
@@ -90,6 +89,29 @@ struct PathComponent
  * Name: [_a-zA-Z][-._a-zA-Z0-9]*
  */
 
+namespace
+{
+// Parsing property names is a profiling hotspot. The regular C
+// library functions interact with the locale and are therefore quite
+// heavyweight. We only support ASCII letters and numbers in property
+// names, so use these simple functions instead.
+
+inline bool isalpha_c(int c)
+{
+    return (c <= 'Z' && c >= 'A') || (c <= 'z' && c >= 'a');
+}
+
+inline bool isdigit_c(int c)
+{
+    return c <= '9' && c >= '0';
+}
+
+inline bool isspecial_c(int c)
+{
+    return  c == '_' ||  c == '-' || c == '.';
+}
+}
+
 template<typename Range>
 inline Range
 parse_name (const SGPropertyNode *node, const Range &path)
@@ -104,14 +126,13 @@ parse_name (const SGPropertyNode *node, const Range &path)
     }
     if (i != max && *i != '/')
       throw std::string("illegal character after . or ..");
-  } else if (isalpha(*i) || *i == '_') {
+  } else if (isalpha_c(*i) || *i == '_') {
     i++;
 
 				// The rules inside a name are a little
 				// less restrictive.
     while (i != max) {
-      if (isalpha(*i) || isdigit(*i) || *i == '_' ||
-	  *i == '-' || *i == '.') {
+        if (isalpha_c(*i) || isdigit_c(*i) || isspecial_c(*i)) {
 	// name += path[i];
       } else if (*i == '[' || *i == '/') {
 	break;
@@ -141,7 +162,6 @@ parse_name (const SGPropertyNode *node, const Range &path)
 // Validate the name of a single node
 inline bool validateName(const std::string& name)
 {
-  using namespace boost;
   if (name.empty())
     return false;
   if (!isalpha(name[0]) && name[0] != '_')
@@ -157,8 +177,8 @@ inline bool validateName(const std::string& name)
   }
   return rv;
 #else
-  return all(make_iterator_range(name.begin(), name.end()),
-             is_alnum() || is_any_of("_-."));
+  return std::all_of(name.begin() + 1, name.end(),
+                     [](int c){ return isalpha_c(c) || isdigit_c(c) ||  isspecial_c(c); });
 #endif
 }
 
@@ -827,7 +847,7 @@ SGPropertyNode::trace_read () const
  * Last used attribute
  * Update as needed when enum Attribute is changed
  */
-const int SGPropertyNode::LAST_USED_ATTRIBUTE = PROTECTED;
+const int SGPropertyNode::LAST_USED_ATTRIBUTE = LISTENER_SAFE;
 
 /**
  * Default constructor: always creates a root node.
@@ -2067,10 +2087,9 @@ SGPropertyNode::getNode (const char * relative_path, bool create)
   return find_node(this, components, 0, create);
 
 #else
-  using namespace boost;
-
-  return find_node(this, make_iterator_range(relative_path, relative_path
-                                             + strlen(relative_path)),
+  return find_node(this,
+                   boost::make_iterator_range(relative_path,
+                                              relative_path + strlen(relative_path)),
                    create);
 #endif
 }
@@ -2086,10 +2105,9 @@ SGPropertyNode::getNode (const char * relative_path, int index, bool create)
   return find_node(this, components, 0, create);
 
 #else
-  using namespace boost;
-
-  return find_node(this, make_iterator_range(relative_path, relative_path
-                                             + strlen(relative_path)),
+  return find_node(this,
+                   boost::make_iterator_range(relative_path,
+                                              relative_path + strlen(relative_path)),
                    create, index);
 #endif
 }
