@@ -33,12 +33,9 @@
 #include <queue>
 #include <utility>
 #include <unordered_map>
+#include <mutex>
 
-#include <boost/bind.hpp>
-#include <boost/foreach.hpp>
 #include <boost/functional/hash.hpp>
-#include <boost/tuple/tuple.hpp>
-#include <boost/tuple/tuple_comparison.hpp>
 
 #include <osg/AlphaFunc>
 #include <osg/BlendFunc>
@@ -77,9 +74,10 @@
 #include <simgear/scene/util/StateAttributeFactory.hxx>
 #include <simgear/structure/OSGUtils.hxx>
 #include <simgear/structure/SGExpression.hxx>
+#include <simgear/props/props_io.hxx>
 #include <simgear/props/vectorPropTemplates.hxx>
-#include <simgear/threads/SGThread.hxx>
-#include <simgear/threads/SGGuard.hxx>
+
+#include <simgear/io/iostreams/sgstream.hxx>
 
 namespace simgear
 {
@@ -97,13 +95,25 @@ void UniformFactoryImpl::reset()
   uniformCache.clear();
 }
 
+// work around the fact osg::Shader::loadShaderFromSourceFile does not
+// respect UTF8 paths, even when OSG_USE_UTF8_FILENAME is set :(
+bool loadShaderFromUTF8File(osg::Shader* shader, const std::string& fileName)
+{
+    sg_ifstream inStream(SGPath::fromUtf8(fileName), std::ios::in | std::ios::binary);
+    if (!inStream.is_open())
+        return false;
+
+    shader->setShaderSource(inStream.read_all());
+    return true;
+}
+
 ref_ptr<Uniform> UniformFactoryImpl::getUniform( Effect * effect,
                                  const string & name,
                                  Uniform::Type uniformType,
                                  SGConstPropertyNode_ptr valProp,
                                  const SGReaderWriterOptions* options )
 {
-	SGGuard<SGMutex> scopeLock(_mutex);
+	std::lock_guard<std::mutex> scopeLock(_mutex);
 	std::string val = "0";
 
 	if (valProp->nChildren() == 0) {
@@ -126,7 +136,7 @@ ref_ptr<Uniform> UniformFactoryImpl::getUniform( Effect * effect,
 		}
 	}
 
-	UniformCacheKey key = boost::make_tuple(name, uniformType, val, effect->getName());
+	UniformCacheKey key = std::make_tuple(name, uniformType, val, effect->getName());
 	ref_ptr<Uniform> uniform = uniformCache[key];
 
     if (uniform.valid()) {
@@ -190,7 +200,7 @@ void UniformFactoryImpl::addListener(DeferredPropertyListener* listener)
 
 void UniformFactoryImpl::updateListeners( SGPropertyNode* propRoot )
 {
-	SGGuard<SGMutex> scopeLock(_mutex);
+	std::lock_guard<std::mutex> scopeLock(_mutex);
 
 	if (deferredListenerList.empty()) return;
 
@@ -245,11 +255,11 @@ int Effect::getGenerator(Effect::Generator what) const
 
 // There should always be a valid technique in an effect.
 
-Technique* Effect::chooseTechnique(RenderInfo* info)
+Technique* Effect::chooseTechnique(RenderInfo* info, const std::string &scheme)
 {
-    BOOST_FOREACH(ref_ptr<Technique>& technique, techniques)
-    {
-        if (technique->valid(info) == Technique::VALID)
+    for (auto& technique : techniques) {
+        if (technique->valid(info) == Technique::VALID &&
+            technique->getScheme() == scheme)
             return technique.get();
     }
     return 0;
@@ -257,16 +267,14 @@ Technique* Effect::chooseTechnique(RenderInfo* info)
 
 void Effect::resizeGLObjectBuffers(unsigned int maxSize)
 {
-    BOOST_FOREACH(const ref_ptr<Technique>& technique, techniques)
-    {
+    for (const auto& technique : techniques) {
         technique->resizeGLObjectBuffers(maxSize);
     }
 }
 
 void Effect::releaseGLObjects(osg::State* state) const
 {
-    BOOST_FOREACH(const ref_ptr<Technique>& technique, techniques)
-    {
+    for (const auto& technique : techniques) {
         technique->releaseGLObjects(state);
     }
 }
@@ -828,7 +836,7 @@ void reload_shaders()
         Shader *shader = sitr->second.get();
         string fileName = SGModelLib::findDataFile(sitr->first.first);
         if (!fileName.empty()) {
-            shader->loadShaderSourceFromFile(fileName);
+            loadShaderFromUTF8File(shader, fileName);
         }
         else
             SG_LOG(SG_INPUT, SG_ALERT, "Could not locate shader: " << fileName);
@@ -869,7 +877,6 @@ void ShaderProgramBuilder::buildAttribute(Effect* effect, Pass* pass,
                                           const SGReaderWriterOptions*
                                           options)
 {
-    using namespace boost;
     if (!isAttributeActive(effect, prop))
         return;
     PropertyList pVertShaders = prop->getChildren("vertex-shader");
@@ -878,12 +885,15 @@ void ShaderProgramBuilder::buildAttribute(Effect* effect, Pass* pass,
     PropertyList pAttributes = prop->getChildren("attribute");
     ProgramKey prgKey;
     std::back_insert_iterator<vector<ShaderKey> > inserter(prgKey.shaders);
-    transform(pVertShaders.begin(), pVertShaders.end(), inserter,
-              boost::bind(makeShaderKey, _1, Shader::VERTEX));
-    transform(pGeomShaders.begin(), pGeomShaders.end(), inserter,
-              boost::bind(makeShaderKey, _1, Shader::GEOMETRY));
-    transform(pFragShaders.begin(), pFragShaders.end(), inserter,
-              boost::bind(makeShaderKey, _1, Shader::FRAGMENT));
+    std::transform(pVertShaders.begin(), pVertShaders.end(), inserter,
+                   [] (SGPropertyNode_ptr& ptr) {
+                       return makeShaderKey(ptr, Shader::VERTEX); });
+    std::transform(pGeomShaders.begin(), pGeomShaders.end(), inserter,
+                   [] (SGPropertyNode_ptr& ptr) {
+                       return makeShaderKey(ptr, Shader::GEOMETRY); });
+    std::transform(pFragShaders.begin(), pFragShaders.end(), inserter,
+                   [] (SGPropertyNode_ptr& ptr) {
+                       return makeShaderKey(ptr, Shader::FRAGMENT); });
     for (PropertyList::iterator itr = pAttributes.begin(),
              e = pAttributes.end();
          itr != e;
@@ -914,10 +924,14 @@ void ShaderProgramBuilder::buildAttribute(Effect* effect, Pass* pass,
     // resolvedProgramMap for a program using those shaders.
     ProgramKey resolvedKey;
     resolvedKey.attributes = prgKey.attributes;
-    BOOST_FOREACH(const ShaderKey& shaderKey, prgKey.shaders)
-    {
-        const string& shaderName = shaderKey.first;
+    for (const auto& shaderKey : prgKey.shaders) {
+        // FIXME orig: const string& shaderName = shaderKey.first;
+        string shaderName = shaderKey.first;
         Shader::Type stype = (Shader::Type)shaderKey.second;
+        if (getPropertyRoot()->getBoolValue("/sim/version/compositor-support", false) &&
+            shaderName.substr(0, shaderName.find("/")) == "Shaders") {
+            shaderName = "Compositor/" + shaderName;
+        }
         string fileName = SGModelLib::findDataFile(shaderName, options);
         if (fileName.empty())
         {
@@ -937,8 +951,7 @@ void ShaderProgramBuilder::buildAttribute(Effect* effect, Pass* pass,
         return;
     }
     program = new Program;
-    BOOST_FOREACH(const ShaderKey& skey, resolvedKey.shaders)
-    {
+    for (const auto& skey : resolvedKey.shaders) {
         const string& fileName = skey.first;
         Shader::Type stype = (Shader::Type)skey.second;
         ShaderMap::iterator sitr = shaderMap.find(skey);
@@ -947,13 +960,13 @@ void ShaderProgramBuilder::buildAttribute(Effect* effect, Pass* pass,
         } else {
             ref_ptr<Shader> shader = new Shader(stype);
 			shader->setName(fileName);
-            if (shader->loadShaderSourceFromFile(fileName)) {
+            if (loadShaderFromUTF8File(shader, fileName)) {
                 program->addShader(shader.get());
                 shaderMap.insert(ShaderMap::value_type(skey, shader));
             }
         }
     }
-    BOOST_FOREACH(const ProgramKey::AttribKey& key, prgKey.attributes) {
+    for (const auto& key : prgKey.attributes) {
         program->addBindAttribLocation(key.first, key.second);
     }
     const SGPropertyNode* pGeometryVerticesOut
@@ -976,6 +989,13 @@ void ShaderProgramBuilder::buildAttribute(Effect* effect, Pass* pass,
                  type);
         program->setParameter(GL_GEOMETRY_OUTPUT_TYPE_EXT, type);
     }
+    PropertyList pUniformBlockBindings
+        = prop->getChildren("uniform-block-binding");
+    for (const auto &pUniformBlockBinding : pUniformBlockBindings) {
+        program->addBindUniformBlock(
+            pUniformBlockBinding->getStringValue("name"),
+            pUniformBlockBinding->getIntValue("index"));
+    }
     programMap.insert(ProgramMap::value_type(prgKey, program));
     resolvedProgramMap.insert(ProgramMap::value_type(resolvedKey, program));
     pass->setAttributeAndModes(program);
@@ -1308,6 +1328,7 @@ void buildTechnique(Effect* effect, const SGPropertyNode* prop,
 {
     Technique* tniq = new Technique;
     effect->techniques.push_back(tniq);
+    tniq->setScheme(prop->getStringValue("scheme"));
     const SGPropertyNode* predProp = prop->getChild("predicate");
     if (!predProp) {
         tniq->setAlwaysValid(true);
@@ -1411,12 +1432,61 @@ bool makeParametersFromStateSet(SGPropertyNode* effectRoot, const StateSet* ss)
     return true;
 }
 
+SGPropertyNode_ptr schemeList;
+
+void mergeSchemesFallbacks(Effect *effect, const SGReaderWriterOptions *options)
+{
+    if (!schemeList) {
+        schemeList = new SGPropertyNode;
+        const string schemes_file("Effects/schemes.xml");
+        string absFileName
+            = SGModelLib::findDataFile(schemes_file, options);
+        if (absFileName.empty()) {
+            SG_LOG(SG_INPUT, SG_ALERT, "Could not find '" << schemes_file << "'");
+            return;
+        }
+        try {
+            readProperties(absFileName, schemeList, 0, true);
+        } catch (sg_io_exception& e) {
+            SG_LOG(SG_INPUT, SG_ALERT, "Error reading '" << schemes_file <<
+                   "': " << e.getFormattedMessage());
+            return;
+        }
+    }
+
+    PropertyList p_schemes = schemeList->getChildren("scheme");
+    for (const auto &p_scheme : p_schemes) {
+        string scheme_name   = p_scheme->getStringValue("name");
+        string fallback_name = p_scheme->getStringValue("fallback");
+        if (scheme_name.empty() || fallback_name.empty())
+            continue;
+        vector<SGPropertyNode_ptr> techniques = effect->root->getChildren("technique");
+        auto it = std::find_if(techniques.begin(), techniques.end(),
+                               [&scheme_name](const SGPropertyNode_ptr &tniq) {
+                                   return tniq->getStringValue("scheme") == scheme_name;
+                               });
+        // Only merge the fallback effect if we haven't found a technique
+        // implementing the scheme
+        if (it == techniques.end()) {
+            ref_ptr<Effect> fallback = makeEffect(fallback_name, false, options);
+            if (fallback) {
+                SGPropertyNode *new_root = new SGPropertyNode;
+                mergePropertyTrees(new_root, effect->root, fallback->root);
+                effect->root = new_root;
+                effect->parametersProp = effect->root->getChild("parameters");
+            }
+        }
+    }
+}
+
 // Walk the techniques property tree, building techniques and
 // passes.
-static SGMutex  realizeTechniques_lock;
+static std::mutex  realizeTechniques_lock;
 bool Effect::realizeTechniques(const SGReaderWriterOptions* options)
 {
-    SGGuard<SGMutex> g(realizeTechniques_lock);
+    std::lock_guard<std::mutex> g(realizeTechniques_lock);
+    if (getPropertyRoot()->getBoolValue("/sim/version/compositor-support", false))
+        mergeSchemesFallbacks(this, options);
 
     if (_isRealized)
         return true;
@@ -1474,7 +1544,7 @@ bool Effect_writeLocalData(const Object& obj, osgDB::Output& fw)
     const Effect& effect = static_cast<const Effect&>(obj);
 
     fw.indent() << "techniques " << effect.techniques.size() << "\n";
-    BOOST_FOREACH(const ref_ptr<Technique>& technique, effect.techniques) {
+    for (const auto& technique : effect.techniques) {
         fw.writeObject(*technique);
     }
     return true;
