@@ -39,10 +39,6 @@
 #include <OpenThreads/Mutex>
 #include <OpenThreads/ScopedLock>
 
-#include <boost/lexical_cast.hpp>
-#include <boost/tuple/tuple.hpp>
-#include <boost/tuple/tuple_comparison.hpp>
-
 #include <simgear/scene/util/OsgMath.hxx>
 #include <simgear/scene/util/SGReaderWriterOptions.hxx>
 #include <simgear/scene/util/SGSceneFeatures.hxx>
@@ -75,9 +71,9 @@ osg::Texture* TextureBuilder::buildFromType(Effect* effect, Pass* pass, const st
     return EffectBuilder<Texture>::buildFromType(effect, pass, type, props, options);
 }
 
-typedef boost::tuple<string, Texture::FilterMode, Texture::FilterMode,
-                     Texture::WrapMode, Texture::WrapMode, Texture::WrapMode,
-                     string, MipMapTuple, ImageInternalFormat> TexTuple;
+typedef std::tuple<string, Texture::FilterMode, Texture::FilterMode,
+                   Texture::WrapMode, Texture::WrapMode, Texture::WrapMode,
+                   string, MipMapTuple, ImageInternalFormat> TexTuple;
 
 EffectNameValue<TexEnv::Mode> texEnvModesInit[] =
 {
@@ -124,10 +120,11 @@ void TextureUnitBuilder::buildAttribute(Effect* effect, Pass* pass,
         const SGPropertyNode* pName = prop->getChild("name");
         if (pName)
             try {
-                unit = boost::lexical_cast<int>(pName->getStringValue());
-            } catch (boost::bad_lexical_cast& lex) {
+                unit = std::stoi(pName->getStringValue());
+            }
+            catch (const std::invalid_argument& ia) {
                 SG_LOG(SG_INPUT, SG_ALERT, "can't decode name as texture unit "
-                       << lex.what());
+                       << ia.what());
             }
     }
     const SGPropertyNode* pType = getEffectPropertyChild(effect, prop, "type");
@@ -264,7 +261,7 @@ TexTuple makeTexTuple(Effect* effect, const SGPropertyNode* props,
 bool setAttrs(const TexTuple& attrs, Texture* tex,
               const SGReaderWriterOptions* options)
 {
-    const string& imageName = attrs.get<0>();
+    const string& imageName = std::get<0>(attrs);
     if (imageName.empty())
         return false;
 
@@ -272,7 +269,7 @@ bool setAttrs(const TexTuple& attrs, Texture* tex,
 
     // load texture for effect
     SGReaderWriterOptions::LoadOriginHint origLOH = options->getLoadOriginHint();
-    if(attrs.get<8>() == ImageInternalFormat::Normalized)
+    if (std::get<8>(attrs) == ImageInternalFormat::Normalized)
         options->setLoadOriginHint(SGReaderWriterOptions::LoadOriginHint::ORIGIN_EFFECTS_NORMALIZED);
     else
         options->setLoadOriginHint(SGReaderWriterOptions::LoadOriginHint::ORIGIN_EFFECTS);
@@ -287,14 +284,14 @@ bool setAttrs(const TexTuple& attrs, Texture* tex,
         image = result.getImage();
     if (image.valid())
     {
-        image = computeMipmap( image.get(), attrs.get<7>() );
+        image = computeMipmap( image.get(), std::get<7>(attrs) );
         tex->setImage(GL_FRONT_AND_BACK, image.get());
         int s = image->s();
         int t = image->t();
         if (s <= t && 32 <= s) {
-            SGSceneFeatures::instance()->setTextureCompression(tex);
+            SGSceneFeatures::instance()->applyTextureCompression(tex);
         } else if (t < s && 32 <= t) {
-            SGSceneFeatures::instance()->setTextureCompression(tex);
+            SGSceneFeatures::instance()->applyTextureCompression(tex);
         }
         tex->setMaxAnisotropy(SGSceneFeatures::instance()->getTextureFilter());
     } else {
@@ -303,11 +300,11 @@ bool setAttrs(const TexTuple& attrs, Texture* tex,
     }
 
     // texture->setDataVariance(osg::Object::STATIC);
-    tex->setFilter(Texture::MIN_FILTER, attrs.get<1>());
-    tex->setFilter(Texture::MAG_FILTER, attrs.get<2>());
-    tex->setWrap(Texture::WRAP_S, attrs.get<3>());
-    tex->setWrap(Texture::WRAP_T, attrs.get<4>());
-    tex->setWrap(Texture::WRAP_R, attrs.get<5>());
+    tex->setFilter(Texture::MIN_FILTER, std::get<1>(attrs));
+    tex->setFilter(Texture::MAG_FILTER, std::get<2>(attrs));
+    tex->setWrap(Texture::WRAP_S, std::get<3>(attrs));
+    tex->setWrap(Texture::WRAP_T, std::get<4>(attrs));
+    tex->setWrap(Texture::WRAP_R, std::get<5>(attrs));
     return true;
 }
 
@@ -356,7 +353,7 @@ namespace
 {
 TextureBuilder::Registrar install1D("1d", new TexBuilder<Texture1D>("1d"));
 TextureBuilder::Registrar install2D("2d", new TexBuilder<Texture2D>("2d"));
-TextureBuilder::Registrar install3D("3d", new TexBuilder<Texture3D>("3d"));
+//TextureBuilder::Registrar install3D("3d", new TexBuilder<Texture3D>("3d"));
 }
 
 class WhiteTextureBuilder : public TextureBuilder
@@ -522,7 +519,7 @@ TextureBuilder::Registrar installLightSprite("light-sprite", new LightSpriteBuil
 }
 
 // Image names for all sides
-typedef boost::tuple<string, string, string, string, string, string> CubeMapTuple;
+typedef std::tuple<string, string, string, string, string, string> CubeMapTuple;
 
 CubeMapTuple makeCubeMapTuple(Effect* effect, const SGPropertyNode* props)
 {
@@ -611,54 +608,54 @@ Texture* CubeMapBuilder::build(Effect* effect, Pass* pass, const SGPropertyNode*
         SGReaderWriterOptions::LoadOriginHint origLOH = wOpts->getLoadOriginHint();
         wOpts->setLoadOriginHint(SGReaderWriterOptions::LoadOriginHint::ORIGIN_EFFECTS);
 #if OSG_VERSION_LESS_THAN(3,4,1)
-        result = osgDB::readImageFile(_tuple.get<0>(), options);
+        result = osgDB::readImageFile(std::get<0>(_tuple), options);
 #else
-        result = osgDB::readRefImageFile(_tuple.get<0>(), options);
+        result = osgDB::readRefImageFile(std::get<0>(_tuple), options);
 #endif
         if(result.success()) {
             osg::Image* image = result.getImage();
             cubeTexture->setImage(TextureCubeMap::POSITIVE_X, image);
         }
 #if OSG_VERSION_LESS_THAN(3,4,1)
-        result = osgDB::readImageFile(_tuple.get<1>(), options);
+        result = osgDB::readImageFile(std::get<1>(_tuple), options);
 #else
-        result = osgDB::readRefImageFile(_tuple.get<1>(), options);
+        result = osgDB::readRefImageFile(std::get<1>(_tuple), options);
 #endif
         if(result.success()) {
             osg::Image* image = result.getImage();
             cubeTexture->setImage(TextureCubeMap::NEGATIVE_X, image);
         }
 #if OSG_VERSION_LESS_THAN(3,4,1)
-        result = osgDB::readImageFile(_tuple.get<2>(), options);
+        result = osgDB::readImageFile(std::get<2>(_tuple), options);
 #else
-        result = osgDB::readRefImageFile(_tuple.get<2>(), options);
+        result = osgDB::readRefImageFile(std::get<2>(_tuple), options);
 #endif
         if(result.success()) {
             osg::Image* image = result.getImage();
             cubeTexture->setImage(TextureCubeMap::POSITIVE_Y, image);
         }
 #if OSG_VERSION_LESS_THAN(3,4,1)
-        result = osgDB::readImageFile(_tuple.get<3>(), options);
+        result = osgDB::readImageFile(std::get<3>(_tuple), options);
 #else
-        result = osgDB::readRefImageFile(_tuple.get<3>(), options);
+        result = osgDB::readRefImageFile(std::get<3>(_tuple), options);
 #endif
         if(result.success()) {
             osg::Image* image = result.getImage();
             cubeTexture->setImage(TextureCubeMap::NEGATIVE_Y, image);
         }
 #if OSG_VERSION_LESS_THAN(3,4,1)
-        result = osgDB::readImageFile(_tuple.get<4>(), options);
+        result = osgDB::readImageFile(std::get<4>(_tuple), options);
 #else
-        result = osgDB::readRefImageFile(_tuple.get<4>(), options);
+        result = osgDB::readRefImageFile(std::get<4>(_tuple), options);
 #endif
         if(result.success()) {
             osg::Image* image = result.getImage();
             cubeTexture->setImage(TextureCubeMap::POSITIVE_Z, image);
         }
 #if OSG_VERSION_LESS_THAN(3,4,1)
-        result = osgDB::readImageFile(_tuple.get<5>(), options);
+        result = osgDB::readImageFile(std::get<5>(_tuple), options);
 #else
-        result = osgDB::readRefImageFile(_tuple.get<5>(), options);
+        result = osgDB::readRefImageFile(std::get<5>(_tuple), options);
 #endif
         if(result.success()) {
             osg::Image* image = result.getImage();
@@ -776,6 +773,96 @@ namespace {
 TextureBuilder::Registrar installCubeMap("cubemap", new CubeMapBuilder);
 }
 
+
+class Texture3DBuilder : public TextureBuilder
+{
+public:
+    Texture* build(Effect* effect, Pass* pass, const SGPropertyNode*,
+                   const SGReaderWriterOptions* options);
+protected:
+    typedef map<TexTuple, observer_ptr<Texture3D> > TexMap;
+    TexMap texMap;
+};
+
+Texture* Texture3DBuilder::build(Effect* effect, Pass* pass,
+                                 const SGPropertyNode* props,
+                                 const SGReaderWriterOptions* options)
+{
+    TexTuple attrs = makeTexTuple(effect, props, options, "3d");
+    typename TexMap::iterator itr = texMap.find(attrs);
+
+    ref_ptr<Texture3D> tex;
+    if ((itr != texMap.end())&&
+        (itr->second.lock(tex)))
+    {
+        return tex.release();
+    }
+
+    tex = new Texture3D;
+
+    const string& imageName = std::get<0>(attrs);
+    if (imageName.empty())
+        return NULL;
+
+    osgDB::ReaderWriter::ReadResult result;
+
+    // load texture for effect
+    SGReaderWriterOptions::LoadOriginHint origLOH = options->getLoadOriginHint();
+    if (std::get<8>(attrs) == ImageInternalFormat::Normalized)
+        options->setLoadOriginHint(SGReaderWriterOptions::LoadOriginHint::ORIGIN_EFFECTS_NORMALIZED);
+    else
+        options->setLoadOriginHint(SGReaderWriterOptions::LoadOriginHint::ORIGIN_EFFECTS);
+#if OSG_VERSION_LESS_THAN(3,4,2)
+    result = osgDB::readImageFile(imageName, options);
+#else
+    result = osgDB::readRefImageFile(imageName, options);
+#endif
+    options->setLoadOriginHint(origLOH);
+    osg::ref_ptr<osg::Image> image;
+    if (result.success())
+        image = result.getImage();
+    if (image.valid())
+    {
+        osg::ref_ptr<osg::Image> image3d = new osg::Image;
+        int size = image->t();
+        int depth = image->s() / image->t();
+        image3d->allocateImage(size, size, depth,
+                               image->getPixelFormat(), image->getDataType());
+
+        for (int i = 0; i < depth; ++i) {
+            osg::ref_ptr<osg::Image> subimage = new osg::Image;
+            subimage->allocateImage(size, size, 1,
+                                    image->getPixelFormat(), image->getDataType());
+            copySubImage(image, size * i, 0, size, size, subimage.get(), 0, 0);
+            image3d->copySubImage(0, 0, i, subimage.get());
+        }
+
+        image3d->setInternalTextureFormat(image->getInternalTextureFormat());
+        image3d = computeMipmap(image3d.get(), std::get<7>(attrs));
+        tex->setImage(image3d.get());
+    } else {
+        SG_LOG(SG_INPUT, SG_ALERT, "failed to load effect texture file " << imageName);
+        return NULL;
+    }
+
+    tex->setFilter(Texture::MIN_FILTER, std::get<1>(attrs));
+    tex->setFilter(Texture::MAG_FILTER, std::get<2>(attrs));
+    tex->setWrap(Texture::WRAP_S, std::get<3>(attrs));
+    tex->setWrap(Texture::WRAP_T, std::get<4>(attrs));
+    tex->setWrap(Texture::WRAP_R, std::get<5>(attrs));
+
+    if (itr == texMap.end())
+        texMap.insert(make_pair(attrs, tex));
+    else
+        itr->second = tex; // update existing, but empty observer
+    return tex.release();
+}
+
+namespace {
+TextureBuilder::Registrar install3D("3d", new Texture3DBuilder);
+}
+
+
 EffectNameValue<TexEnvCombine::CombineParam> combineParamInit[] =
 {
     {"replace", TexEnvCombine::REPLACE},
