@@ -25,8 +25,6 @@
 #include <cstring>
 #include <cassert>
 
-#include <boost/bind.hpp>
-
 #include <osg/Version>
 #include <osg/Geode>
 #include <osg/MatrixTransform>
@@ -50,6 +48,7 @@
 #include "animation.hxx"
 #include "particles.hxx"
 #include "model.hxx"
+#include "SGLight.hxx"
 #include "SGText.hxx"
 #include "SGMaterialAnimation.hxx"
 
@@ -94,7 +93,7 @@ SGReaderWriterXML::readNode(const std::string& name,
         int num_anims;
         std::tie(num_anims, result) = sgLoad3DModel_internal(p, options);
     } catch (const sg_exception &t) {
-        SG_LOG(SG_INPUT, SG_ALERT, "Failed to load model: " << t.getFormattedMessage()
+        SG_LOG(SG_IO, SG_DEV_ALERT, "Failed to load model: " << t.getFormattedMessage()
           << "\n\tfrom:" << fileName);
         result=new osg::Node;
     }
@@ -196,10 +195,10 @@ void makeEffectAnimations(PropertyList& animation_nodes,
 
         }
     }
-    animation_nodes.erase(remove_if(animation_nodes.begin(),
-                                    animation_nodes.end(),
-                                    !boost::bind(&SGPropertyNode_ptr::valid,
-                                                 _1)),
+    animation_nodes.erase(std::remove_if(animation_nodes.begin(),
+                                         animation_nodes.end(),
+                                         [] (const SGPropertyNode_ptr& ptr) {
+                                             return !ptr.valid(); }),
                           animation_nodes.end());
 }
 }
@@ -263,7 +262,7 @@ sgLoad3DModel_internal(const SGPath& path,
                        SGPropertyNode *overlay)
 {
     if (!path.exists()) {
-      SG_LOG(SG_INPUT, SG_ALERT, "Failed to load file: \"" << path << "\"");
+      SG_LOG(SG_IO, SG_DEV_ALERT, "Failed to load file: \"" << path << "\"");
       return std::make_tuple(0, (osg::Node *) NULL);
     }
 
@@ -292,7 +291,7 @@ sgLoad3DModel_internal(const SGPath& path,
        try {
             readProperties(modelpath, props);
         } catch (const sg_exception &t) {
-            SG_LOG(SG_INPUT, SG_ALERT, "Failed to load xml: "
+            SG_LOG(SG_IO, SG_DEV_ALERT, "Failed to load xml: "
                    << t.getFormattedMessage());
             throw;
         }
@@ -337,12 +336,12 @@ sgLoad3DModel_internal(const SGPath& path,
         if (!texturepath.extension().empty())
             texturepath = texturepath.dir();
 
-        options->setDatabasePath(texturepath.local8BitStr());
+        options->setDatabasePath(texturepath.utf8Str());
         osgDB::ReaderWriter::ReadResult modelResult;
 #if OSG_VERSION_LESS_THAN(3,4,1)
-        modelResult = osgDB::readNodeFile(modelpath.local8BitStr(), options.get());
+        modelResult = osgDB::readNodeFile(modelpath.utf8Str(), options.get());
 #else
-        modelResult = osgDB::readRefNodeFile(modelpath.local8BitStr(), options.get());
+        modelResult = osgDB::readRefNodeFile(modelpath.utf8Str(), options.get());
 #endif
         if (!modelResult.validNode())
             throw sg_io_exception("Failed to load 3D model:" + modelResult.message(),
@@ -413,7 +412,7 @@ sgLoad3DModel_internal(const SGPath& path,
           NULL, modelDir);
 
         if (submodelPath.isNull()) {
-          SG_LOG(SG_INPUT, SG_ALERT, "Failed to load file: \"" << subPathStr << "\"");
+          SG_LOG(SG_IO, SG_DEV_ALERT, "Failed to load file: \"" << subPathStr << "\"");
           continue;
         }
 
@@ -432,7 +431,7 @@ sgLoad3DModel_internal(const SGPath& path,
                                               sub_props->getNode("overlay"));
             animationcount += num_anims;
         } catch (const sg_exception &t) {
-            SG_LOG(SG_INPUT, SG_ALERT, "Failed to load submodel: " << t.getFormattedMessage()
+            SG_LOG(SG_IO, SG_DEV_ALERT, "Failed to load submodel: " << t.getFormattedMessage()
               << "\n\tfrom:" << t.getOrigin());
             continue;
         }
@@ -483,7 +482,7 @@ sgLoad3DModel_internal(const SGPath& path,
         // Load panels
         vector<SGPropertyNode_ptr> panel_nodes = props->getChildren("panel");
         for (unsigned i = 0; i < panel_nodes.size(); i++) {
-            SG_LOG(SG_INPUT, SG_DEBUG, "Loading a panel");
+            SG_LOG(SG_IO, SG_DEBUG, "Loading a panel");
             osg::ref_ptr<osg::Node> panel = load_panel(panel_nodes[i]);
             if (panel_nodes[i]->hasValue("name"))
                 panel->setName(panel_nodes[i]->getStringValue("name"));
@@ -502,7 +501,7 @@ sgLoad3DModel_internal(const SGPath& path,
                 if (!texturepath.extension().empty())
                     texturepath = texturepath.dir();
 
-                options2->setDatabasePath(texturepath.local8BitStr());
+                options2->setDatabasePath(texturepath.utf8Str());
             }
             group->addChild(Particles::appendParticles(particle_nodes[i],
                                                        prop_root,
@@ -518,6 +517,14 @@ sgLoad3DModel_internal(const SGPath& path,
                         options.get()));
     }
 
+    std::vector<SGPropertyNode_ptr> light_nodes;
+    light_nodes = props->getChildren("light");
+    for (unsigned i = 0; i < light_nodes.size(); ++i) {
+        group->addChild(SGLight::appendLight(light_nodes[i],
+                                             prop_root,
+                                             options.get()));
+    }
+
     PropertyList effect_nodes = props->getChildren("effect");
     PropertyList animation_nodes = props->getChildren("animation");
 
@@ -535,7 +542,7 @@ sgLoad3DModel_internal(const SGPath& path,
         group = static_cast<Group*>(modelWithEffects.get());
     }
 
-    simgear::SGTransientModelData modelData(group.get(), prop_root, options.get(), path.local8BitStr());
+    simgear::SGTransientModelData modelData(group.get(), prop_root, options.get(), path.utf8Str());
 
     for (unsigned i = 0; i < animation_nodes.size(); ++i) {
         if (previewMode && animation_nodes[i]->hasChild("nopreview")) {
