@@ -426,6 +426,7 @@ struct DoDrawArraysVisitor : public osg::NodeVisitor {
 
 SGAnimation::SGAnimation(simgear::SGTransientModelData &modelData) :
   osg::NodeVisitor(osg::NodeVisitor::TRAVERSE_ALL_CHILDREN),
+  _modelData(modelData),
   _found(false),
   _configNode(modelData.getConfigNode()),
   _modelRoot(modelData.getModelRoot())
@@ -454,8 +455,8 @@ SGAnimation::~SGAnimation()
       }
       if (!info.empty())
       {
-          SG_LOG(SG_IO, SG_ALERT, "Could not find at least one of the following"
-                  " objects for animation: " << info);
+          SG_LOG(SG_IO, SG_DEV_ALERT, "Could not find at least one of the following"
+                  " objects for animation: " << info << " in file: " << _modelData.getPath());
       }
   }
 }
@@ -671,7 +672,7 @@ public:
         _group(0)
     {
         if (name.empty())
-            SG_LOG(SG_IO, SG_WARN, "FindGroupVisitor: empty name provided");
+            SG_LOG(SG_IO, SG_DEV_WARN, "FindGroupVisitor: empty name provided");
     }
 
     osg::Group* getGroup() const
@@ -693,7 +694,7 @@ public:
             SG_LOG
             (
                 SG_IO,
-                SG_WARN,
+                SG_DEV_WARN,
                 "FindGroupVisitor: name not unique '" << _name << "'"
             );
     }
@@ -771,11 +772,13 @@ bool SGAnimation::setCenterAndAxisFromObject(osg::Node *rootNode, SGVec3d& cente
                      */
                     object_group->setNodeMask(0);
                 }
-                else
-                    SG_LOG(SG_INPUT, SG_ALERT, "Could not find a valid line segment for animation:  " << axis_object_name);
+                else {
+                    SG_LOG(SG_IO, SG_DEV_ALERT, "Could not find a valid line segment for animation:  " << axis_object_name << " in file: " << _modelData.getPath());
+                }
+            }
+            else if (can_warn) {
+                SG_LOG(SG_IO, SG_DEV_ALERT, "Could not find at least one of the following objects for axis animation: " << axis_object_name << " in file: " << _modelData.getPath());
             }
-            else if (can_warn)
-                SG_LOG(SG_INPUT, SG_ALERT, "Could not find at least one of the following objects for axis animation: " << axis_object_name);
         }
         if (axisSegment)
         {
@@ -2391,12 +2394,13 @@ SGTexTransformAnimation::createAnimationGroup(osg::Group& parent)
         appendTexRotate(*transformConfigs[i], updateCallback);
       else if (subtype == "textrapezoid")
         appendTexTrapezoid(*transformConfigs[i], updateCallback);
-      else
-        SG_LOG(SG_INPUT, SG_ALERT,
-               "Ignoring unknown texture transform subtype");
+      else {
+        SG_LOG(SG_IO, SG_DEV_ALERT,
+               "Ignoring unknown texture transform subtype in file: " << _modelData.getPath());
+      }
     }
   } else {
-    SG_LOG(SG_INPUT, SG_ALERT, "Ignoring unknown texture transform type");
+    SG_LOG(SG_IO, SG_DEV_ALERT, "Ignoring unknown texture transform type in file: " << _modelData.getPath());
   }
 
   texMat->setUpdateCallback(updateCallback);
