@@ -95,6 +95,13 @@ static SGBucket bucketIndexFromFileName(const std::string& fileName)
   return SGBucket(index);
 }
 
+/**
+ * callback per STG token, with access synced by a lock.
+ */
+using TokenCallbackMap = std::map<std::string,ReaderWriterSTG::STGObjectCallback>;
+static TokenCallbackMap globalStgObjectCallbacks = {};
+static OpenThreads::Mutex globalStgObjectCallbackLock;
+
 struct ReaderWriterSTG::_ModelBin {
     struct _Object {
         SGPath _errorLocation;
@@ -269,7 +276,7 @@ struct ReaderWriterSTG::_ModelBin {
 
         SGPath path = filePath;
         path.append(".."); path.append(".."); path.append("..");
-        sharedOptions->getDatabasePathList().push_back(path.local8BitStr());
+        sharedOptions->getDatabasePathList().push_back(path.utf8Str());
 
         // ensure Models directory synced via TerraSync is searched before the copy in
         // FG_ROOT, so that updated models can be used.
@@ -365,7 +372,7 @@ struct ReaderWriterSTG::_ModelBin {
 
         SG_LOG(SG_TERRAIN, SG_INFO, "Loading stg file " << absoluteFileName);
 
-        std::string filePath = osgDB::getFilePath(absoluteFileName.local8BitStr());
+        std::string filePath = osgDB::getFilePath(absoluteFileName.utf8Str());
 
         // Bucket provides a consistent seed
         // so we have consistent set of pseudo-random numbers for each STG file
@@ -413,7 +420,7 @@ struct ReaderWriterSTG::_ModelBin {
                     _Object obj;
                     obj._errorLocation = absoluteFileName;
                     obj._token = token;
-                    obj._name = path.local8BitStr();
+                    obj._name = path.utf8Str();
                     obj._options = staticOptions(filePath, options);
                     _objectList.push_back(obj);
                 }
@@ -423,7 +430,7 @@ struct ReaderWriterSTG::_ModelBin {
                     _Object obj;
                     obj._errorLocation = absoluteFileName;
                     obj._token = token;
-                    obj._name = path.local8BitStr();
+                    obj._name = path.utf8Str();
                     obj._options = staticOptions(filePath, options);
                     _objectList.push_back(obj);
                 }
@@ -530,14 +537,32 @@ struct ReaderWriterSTG::_ModelBin {
                     _objectStaticList.push_back(obj);
                 } else if (token == BUILDING_LIST) {
                   _BuildingList buildinglist;
-                  buildinglist._filename = path.local8BitStr();
+                  buildinglist._filename = path.utf8Str();
                   in >> buildinglist._material_name >> buildinglist._lon >> buildinglist._lat >> buildinglist._elev;
                   checkInsideBucket(absoluteFileName, buildinglist._lon, buildinglist._lat);
                   _buildingListList.push_back(buildinglist);
                   //SG_LOG(SG_TERRAIN, SG_ALERT, "Building list: " << buildinglist._filename << " " << buildinglist._material_name << " " << buildinglist._lon << " " << buildinglist._lat);
                 } else {
-                    SG_LOG( SG_TERRAIN, SG_ALERT, absoluteFileName
-                            << ": Unknown token '" << token << "'" );
+                    // Check registered callback for token. Keep lock until callback completed to make sure it will not be
+                    // executed after a thread successfully executed removeSTGObjectHandler()
+                    {
+                        OpenThreads::ScopedLock<OpenThreads::Mutex> lock(globalStgObjectCallbackLock);
+                        STGObjectCallback callback = globalStgObjectCallbacks[token];
+
+                        if (callback != nullptr) {
+                            _ObjectStatic obj;
+                            // pitch and roll are not common, so passed in "restofline" only
+                            in >> obj._lon >> obj._lat >> obj._elev >> obj._hdg;
+                            string_list restofline;
+                            std::string buf;
+                            while (in >> buf) {
+                                restofline.push_back(buf);
+                            }
+                            callback(token,name, SGGeod::fromDegM(obj._lon, obj._lat, obj._elev), obj._hdg,restofline);
+                        } else {
+                            SG_LOG( SG_TERRAIN, SG_ALERT, absoluteFileName << ": Unknown token '" << token << "'" );
+                        }
+                    }
                 }
             }
         }
@@ -711,4 +736,16 @@ ReaderWriterSTG::readNode(const std::string& fileName, const osgDB::Options* opt
     return modelBin.load(bucket, options);
 }
 
+
+void ReaderWriterSTG::setSTGObjectHandler(const std::string &token, STGObjectCallback callback)
+{
+    OpenThreads::ScopedLock<OpenThreads::Mutex> lock(globalStgObjectCallbackLock);
+    globalStgObjectCallbacks[token] = callback;
+}
+
+void ReaderWriterSTG::removeSTGObjectHandler(const std::string &token, STGObjectCallback callback)
+{
+    OpenThreads::ScopedLock<OpenThreads::Mutex> lock(globalStgObjectCallbackLock);
+    globalStgObjectCallbacks.erase(token);
+}
 }
