@@ -27,7 +27,6 @@
 #include "pt_lights.hxx"
 
 #include <map>
-#include <boost/tuple/tuple_comparison.hpp>
 
 #include <osg/Array>
 #include <osg/Geometry>
@@ -78,7 +77,7 @@ static Mutex lightMutex;
 
 namespace
 {
-typedef boost::tuple<float, osg::Vec3, float, float, bool> PointParams;
+typedef std::tuple<float, osg::Vec3, float, float, bool> PointParams;
 typedef std::map<PointParams, observer_ptr<Effect> > EffectMap;
 
 EffectMap effectMap;
@@ -153,41 +152,49 @@ SGLightFactory::getLightDrawable(const SGLightBin::Light& light)
 }
 
 osg::Drawable*
-SGLightFactory::getLightDrawable(const SGDirectionalLightBin::Light& light)
+SGLightFactory::getLightDrawable(const SGDirectionalLightBin::Light& light, bool useTriangles)
 {
   osg::Vec3Array* vertices = new osg::Vec3Array;
   osg::Vec4Array* colors = new osg::Vec4Array;
 
-  SGVec4f visibleColor(light.color);
-  SGVec4f invisibleColor(visibleColor[0], visibleColor[1],
-                         visibleColor[2], 0);
-  SGVec3f normal = normalize(light.normal);
-  SGVec3f perp1 = perpendicular(normal);
-  SGVec3f perp2 = cross(normal, perp1);
-  SGVec3f position = light.position;
-  vertices->push_back(toOsg(position));
-  vertices->push_back(toOsg(position + perp1));
-  vertices->push_back(toOsg(position + perp2));
-  colors->push_back(toOsg(visibleColor));
-  colors->push_back(toOsg(invisibleColor));
-  colors->push_back(toOsg(invisibleColor));
+  if (useTriangles) {
+    SGVec4f visibleColor(light.color);
+    SGVec4f invisibleColor(visibleColor[0], visibleColor[1],
+                           visibleColor[2], 0);
+    SGVec3f normal = normalize(light.normal);
+    SGVec3f perp1 = perpendicular(normal);
+    SGVec3f perp2 = cross(normal, perp1);
+    SGVec3f position = light.position;
+    vertices->push_back(toOsg(position));
+    vertices->push_back(toOsg(position + perp1));
+    vertices->push_back(toOsg(position + perp2));
+    colors->push_back(toOsg(visibleColor));
+    colors->push_back(toOsg(invisibleColor));
+    colors->push_back(toOsg(invisibleColor));
 
-  osg::Geometry* geometry = new osg::Geometry;
-  geometry->setDataVariance(osg::Object::STATIC);
-  geometry->setVertexArray(vertices);
-  geometry->setNormalBinding(osg::Geometry::BIND_OFF);
-  geometry->setColorArray(colors);
-  geometry->setColorBinding(osg::Geometry::BIND_PER_VERTEX);
+    osg::Geometry* geometry = new osg::Geometry;
+    geometry->setDataVariance(osg::Object::STATIC);
+    geometry->setVertexArray(vertices);
+    geometry->setNormalBinding(osg::Geometry::BIND_OFF);
+    geometry->setColorArray(colors);
+    geometry->setColorBinding(osg::Geometry::BIND_PER_VERTEX);
 
-  // Enlarge the bounding box to avoid such light nodes being victim to
-  // small feature culling.
-  geometry->setComputeBoundingBoxCallback(new SGEnlargeBoundingBox(1));
+    // Enlarge the bounding box to avoid such light nodes being victim to
+    // small feature culling.
+    geometry->setComputeBoundingBoxCallback(new SGEnlargeBoundingBox(1));
 
-  osg::DrawArrays* drawArrays;
-  drawArrays = new osg::DrawArrays(osg::PrimitiveSet::TRIANGLES,
-                                   0, vertices->size());
-  geometry->addPrimitiveSet(drawArrays);
-  return geometry;
+    osg::DrawArrays* drawArrays;
+    drawArrays = new osg::DrawArrays(osg::PrimitiveSet::TRIANGLES,
+                                     0, vertices->size());
+    geometry->addPrimitiveSet(drawArrays);
+    return geometry;
+  } else {
+    // Workaround for driver issue where point sprites cannot be triangles,
+    // (which we use to provide a sensible normal).  So fall back to a simple
+    // non-directional point sprite.
+    const SGLightBin::Light nonDirectionalLight = SGLightBin::Light(light.position, light.color);
+    return getLightDrawable(nonDirectionalLight);
+  }
 }
 
 namespace
@@ -214,8 +221,7 @@ SGLightFactory::getLights(const SGLightBin& lights, unsigned inc, float alphaOff
   geometry->setDataVariance(osg::Object::STATIC);
   geometry->setVertexArray(vertices);
   geometry->setNormalBinding(osg::Geometry::BIND_OFF);
-  geometry->setColorArray(colors);
-  geometry->setColorBinding(osg::Geometry::BIND_PER_VERTEX);
+  geometry->setColorArray(colors, osg::Array::BIND_PER_VERTEX);
 
   osg::DrawArrays* drawArrays;
   drawArrays = new osg::DrawArrays(osg::PrimitiveSet::POINTS,
@@ -275,7 +281,7 @@ SGLightFactory::getLights(const SGDirectionalLightBin& lights)
   //stateSet->setMode(GL_LIGHTING, osg::StateAttribute::OFF);
 
   osg::DrawArrays* drawArrays;
-  drawArrays = new osg::DrawArrays(osg::PrimitiveSet::POINTS,
+  drawArrays = new osg::DrawArrays(osg::PrimitiveSet::TRIANGLES,
                                    0, vertices->size());
   geometry->addPrimitiveSet(drawArrays);
   return geometry;
@@ -368,20 +374,54 @@ SGLightFactory::getSequenced(const SGDirectionalLightBin& lights, const SGReader
   if (lights.getNumLights() <= 0)
     return 0;
 
+  static SGSceneFeatures* sceneFeatures = SGSceneFeatures::instance();
+  bool useTriangles = sceneFeatures->getEnableTriangleDirectionalLights();
+
   // generate a repeatable random seed
   sg_srandom(unsigned(lights.getLight(0).position[0]));
-  float flashTime = 2e-2 + 5e-3*sg_random();
+  float flashTime = 0.065 + 0.003 * sg_random();
   osg::Sequence* sequence = new osg::Sequence;
   sequence->setDefaultTime(flashTime);
-  Effect* effect = getLightEffect(10.0f, osg::Vec3(1.0, 0.0001, 0.00000001),
-                                  6.0f, 10.0f, true, options);
+  Effect* effect = getLightEffect(24.0f, osg::Vec3(1.0, 0.0001, 0.000001),
+                                  1.0f, 24.0f, true, options);
   for (int i = lights.getNumLights() - 1; 0 <= i; --i) {
     EffectGeode* egeode = new EffectGeode;
     egeode->setEffect(effect);
-    egeode->addDrawable(getLightDrawable(lights.getLight(i)));
+    egeode->addDrawable(getLightDrawable(lights.getLight(i), useTriangles));
     sequence->addChild(egeode, flashTime);
   }
-  sequence->addChild(new osg::Group, 1 + 1e-1*sg_random());
+  sequence->addChild(new osg::Group, 1.9 + (0.1 * sg_random()) - (lights.getNumLights() * flashTime));
+  sequence->setInterval(osg::Sequence::LOOP, 0, -1);
+  sequence->setDuration(1.0f, -1);
+  sequence->setMode(osg::Sequence::START);
+  sequence->setSync(true);
+  return sequence;
+}
+
+osg::Node*
+SGLightFactory::getReil(const SGDirectionalLightBin& lights, const SGReaderWriterOptions* options)
+{
+  if (lights.getNumLights() <= 0)
+    return 0;
+
+  static SGSceneFeatures* sceneFeatures = SGSceneFeatures::instance();
+  bool useTriangles = sceneFeatures->getEnableTriangleDirectionalLights();
+
+  // generate a repeatable random seed
+  sg_srandom(unsigned(lights.getLight(0).position[0]));
+  float flashTime = 0.065 + 0.003 * sg_random();
+  osg::Sequence* sequence = new osg::Sequence;
+  sequence->setDefaultTime(flashTime);
+  Effect* effect = getLightEffect(24.0f, osg::Vec3(1.0, 0.0001, 0.000001),
+                                  1.0f, 24.0f, true, options);
+  EffectGeode* egeode = new EffectGeode;
+  egeode->setEffect(effect);
+
+  for (int i = lights.getNumLights() - 1; 0 <= i; --i) {
+    egeode->addDrawable(getLightDrawable(lights.getLight(i), useTriangles));
+  }
+  sequence->addChild(egeode, flashTime);
+  sequence->addChild(new osg::Group, 1.9 + 0.1 * sg_random() - flashTime);
   sequence->setInterval(osg::Sequence::LOOP, 0, -1);
   sequence->setDuration(1.0f, -1);
   sequence->setMode(osg::Sequence::START);
@@ -397,11 +437,11 @@ SGLightFactory::getOdal(const SGLightBin& lights, const SGReaderWriterOptions* o
 
   // generate a repeatable random seed
   sg_srandom(unsigned(lights.getLight(0).position[0]));
-  float flashTime = 2e-2 + 5e-3*sg_random();
+  float flashTime = 0.065 + 0.003 * sg_random();
   osg::Sequence* sequence = new osg::Sequence;
   sequence->setDefaultTime(flashTime);
-  Effect* effect = getLightEffect(10.0f, osg::Vec3(1.0, 0.0001, 0.00000001),
-                                  6.0, 10.0, false, options);
+  Effect* effect = getLightEffect(20.0f, osg::Vec3(1.0, 0.0001, 0.000001),
+                                  1.0f, 20.0f, false, options);
   // centerline lights
   for (int i = lights.getNumLights() - 1; i >= 2; i--) {
     EffectGeode* egeode = new EffectGeode;
@@ -409,18 +449,18 @@ SGLightFactory::getOdal(const SGLightBin& lights, const SGReaderWriterOptions* o
     egeode->addDrawable(getLightDrawable(lights.getLight(i)));
     sequence->addChild(egeode, flashTime);
   }
+  // add extra empty group for a break
+  sequence->addChild(new osg::Group, 4 * flashTime);
   // runway end lights
-  osg::Group* group = new osg::Group;
+  EffectGeode* egeode = new EffectGeode;
+  egeode->setEffect(effect);
   for (unsigned i = 0; i < 2; ++i) {
-    EffectGeode* egeode = new EffectGeode;
-    egeode->setEffect(effect);
     egeode->addDrawable(getLightDrawable(lights.getLight(i)));
-    group->addChild(egeode);
   }
-  sequence->addChild(group, flashTime);
+  sequence->addChild(egeode, flashTime);
 
   // add an extra empty group for a break
-  sequence->addChild(new osg::Group, 2 + 1e-1*sg_random());
+  sequence->addChild(new osg::Group, 1.9 + (0.1 * sg_random()) - ((lights.getNumLights() + 2) * flashTime));
   sequence->setInterval(osg::Sequence::LOOP, 0, -1);
   sequence->setDuration(1.0f, -1);
   sequence->setMode(osg::Sequence::START);
@@ -436,24 +476,26 @@ SGLightFactory::getHoldShort(const SGDirectionalLightBin& lights, const SGReader
   if (lights.getNumLights() < 2)
     return 0;
 
+  static SGSceneFeatures* sceneFeatures = SGSceneFeatures::instance();
+  bool useTriangles = sceneFeatures->getEnableTriangleDirectionalLights();
+
   sg_srandom(unsigned(lights.getLight(0).position[0]));
-  float flashTime = 1 + 0.1 * sg_random();
+  float flashTime = 0.9 + 0.2 * sg_random();
   osg::Sequence* sequence = new osg::Sequence;
 
   // start with lights off
-  sequence->addChild(new osg::Group, flashTime);
+  sequence->addChild(new osg::Group, 0.2);
   // ...and increase the lights in steps
-  for (int i = 2; i < 7; i+=2) {
-      Effect* effect = getLightEffect(i, osg::Vec3(1, 0.001, 0.000002),
-                                      0.0f, i, true, options);
+  for (int i = 0; i < 5; i++) {
+      Effect* effect = getLightEffect(12.0f + i, osg::Vec3(1, 0.001, 0.0002),
+                                      1.0f, 12.0f + i, true, options);
       EffectGeode* egeode = new EffectGeode;
+      egeode->setEffect(effect);
       for (unsigned int j = 0; j < lights.getNumLights(); ++j) {
-          egeode->addDrawable(getLightDrawable(lights.getLight(j)));
-          egeode->setEffect(effect);
+          egeode->addDrawable(getLightDrawable(lights.getLight(j), useTriangles));
       }
-      sequence->addChild(egeode, (i==6) ? flashTime : 0.1);
+      sequence->addChild(egeode, (i==4) ? flashTime : 0.1);
   }
-
   sequence->setInterval(osg::Sequence::SWING, 0, -1);
   sequence->setDuration(1.0f, -1);
   sequence->setMode(osg::Sequence::START);
@@ -468,17 +510,20 @@ SGLightFactory::getGuard(const SGDirectionalLightBin& lights, const SGReaderWrit
   if (lights.getNumLights() < 2)
     return 0;
 
+  static SGSceneFeatures* sceneFeatures = SGSceneFeatures::instance();
+  bool useTriangles = sceneFeatures->getEnableTriangleDirectionalLights();
+
   // generate a repeatable random seed
   sg_srandom(unsigned(lights.getLight(0).position[0]));
-  float flashTime = 1.0f + 1*sg_random();
+  float flashTime = 0.9 + 0.2 * sg_random();
   osg::Sequence* sequence = new osg::Sequence;
   sequence->setDefaultTime(flashTime);
-  Effect* effect = getLightEffect(10.0f, osg::Vec3(1.0, 0.001, 0.000002),
-                                  0.0f, 8.0f, true, options);
+  Effect* effect = getLightEffect(16.0f, osg::Vec3(1.0, 0.001, 0.0002),
+                                  1.0f, 16.0f, true, options);
   for (unsigned int i = 0; i < lights.getNumLights(); ++i) {
     EffectGeode* egeode = new EffectGeode;
     egeode->setEffect(effect);
-    egeode->addDrawable(getLightDrawable(lights.getLight(i)));
+    egeode->addDrawable(getLightDrawable(lights.getLight(i), useTriangles));
     sequence->addChild(egeode, flashTime);
   }
   sequence->setInterval(osg::Sequence::LOOP, 0, -1);
