@@ -21,6 +21,7 @@
 #include <simgear_config.h>
 
 #include <algorithm>
+#include <cassert>
 
 #include <simgear/debug/logstream.hxx>
 #include <simgear/timing/timestamp.hxx>
@@ -45,8 +46,10 @@ using State = SGSubsystem::State;
 
 SGSubsystemTimingCb SGSubsystem::reportTimingCb = NULL;
 void* SGSubsystem::reportTimingUserData = NULL;
+bool SGSubsystem::reportTimingStatsRequest = false;
+int SGSubsystem::maxTimePerFrame_ms = 7;
 
-SGSubsystem::SGSubsystem ()
+SGSubsystem::SGSubsystem () : _executionTime(0), _lastExecutionTime(0)
 {
 }
 
@@ -135,28 +138,28 @@ void SGSubsystem::stamp(const string& name)
 
 void SGSubsystem::set_name(const std::string &n)
 {
-    assert(_name.empty());
-    _name = n;
+    assert(_subsystemId.empty());
+    _subsystemId = n;
 }
 
-std::string SGSubsystem::typeName() const
+std::string SGSubsystem::subsystemClassId() const
 {
-    auto pos = _name.find(SUBSYSTEM_NAME_SEPARATOR);
+    auto pos = _subsystemId.find(SUBSYSTEM_NAME_SEPARATOR);
     if (pos == std::string::npos) {
-        return _name;
+        return _subsystemId;
     }
     
-    return _name.substr(0, pos);
+    return _subsystemId.substr(0, pos);
 }
 
-std::string SGSubsystem::instanceName() const
+std::string SGSubsystem::subsystemInstanceId() const
 {
-    auto pos = _name.find(SUBSYSTEM_NAME_SEPARATOR);
+    auto pos = _subsystemId.find(SUBSYSTEM_NAME_SEPARATOR);
     if (pos == std::string::npos) {
         return {};
     }
     
-    return _name.substr(pos+1);
+    return _subsystemId.substr(pos+1);
 }
 
 void SGSubsystem::set_group(SGSubsystemGroup* group)
@@ -172,7 +175,7 @@ SGSubsystemGroup* SGSubsystem::get_group() const
 SGSubsystemMgr* SGSubsystem::get_manager() const
 {
     if (!get_group())
-        throw sg_exception("SGSubsystem::get_manager: subsystem " + name() + " has no group");
+        throw sg_exception("SGSubsystem::get_manager: subsystem " + subsystemId() + " has no group");
     return get_group()->get_manager();
 }
 
@@ -202,16 +205,20 @@ std::string SGSubsystem::nameForState(State s)
 class SGSubsystemGroup::Member
 {    
 private:
-    Member (const Member &member);
+    Member(const Member &member);
 public:
-    Member ();
+    Member();
     ~Member ();
     
     void update (double delta_time_sec);
 
     void reportTiming(void) { if (reportTimingCb) reportTimingCb(reportTimingUserData, name, &timeStat); }
-    void updateExecutionTime(double time) { timeStat += time;}
+    void reportTimingStats(TimerStats *_lastValues) {
+        if (subsystem)
+            subsystem->reportTimingStats(_lastValues);
+    }
 
+    void updateExecutionTime(double time) { timeStat += time;}
     SampleStatistic timeStat;
     std::string name;
     SGSubsystemRef subsystem;
@@ -220,14 +227,16 @@ public:
     bool collectTimeStats;
     int exceptionCount;
     int initTime;
+
+    void mergeTimerStats(SGSubsystem::TimerStats &stats);
 };
 
 
 
-SGSubsystemGroup::SGSubsystemGroup () :
-  _fixedUpdateTime(-1.0),
-  _updateTimeRemainder(0.0),
-  _initPosition(-1)
+SGSubsystemGroup::SGSubsystemGroup() :
+    _fixedUpdateTime(-1.0),
+    _updateTimeRemainder(0.0),
+    _initPosition(-1)
 {
 }
 
@@ -281,7 +290,7 @@ SGSubsystemGroup::incrementalInit()
         notifyDidChange(m->subsystem, State::INIT);
         ++_initPosition;
         
-        if (_initPosition < _members.size()) {
+        if (_initPosition < static_cast<int>(_members.size())) {
             // start init of the next one
             notifyWillChange( _members[_initPosition]->subsystem, State::INIT);
         }
@@ -377,21 +386,145 @@ SGSubsystemGroup::update (double delta_time_sec)
 
     const bool recordTime = (reportTimingCb != nullptr);
     SGTimeStamp timeStamp;
+    TimerStats lvTimerStats(_timerStats);
+    TimerStats overrunItems;
+    bool overrun = false;
+
+    SGTimeStamp outerTimeStamp;
+    outerTimeStamp.stamp();
     while (loopCount-- > 0) {
         for (auto member : _members) {
-          if (recordTime)
-              timeStamp = SGTimeStamp::now();
 
+          timeStamp.stamp();
+          if (member->subsystem->_timerStats.size()) {
+              member->subsystem->_lastTimerStats.clear();
+              member->subsystem->_lastTimerStats.insert(member->subsystem->_timerStats.begin(), member->subsystem->_timerStats.end());
+          }
           member->update(delta_time_sec); // indirect call
+          if (member->name.size())
+              _timerStats[member->name] += timeStamp.elapsedMSec() / 1000.0;
 
           if (recordTime && reportTimingCb) {
-              timeStamp = SGTimeStamp::now() - timeStamp;
-              member->updateExecutionTime(timeStamp.toUSecs());
+              member->updateExecutionTime(timeStamp.elapsedMSec()*1000);
+              if (timeStamp.elapsedMSec() > SGSubsystemMgr::maxTimePerFrame_ms) {
+                  overrunItems[member->name] += timeStamp.elapsedMSec();
+                  overrun = true;
+              }
           }
       }
     } // of multiple update loop
+    _lastExecutionTime = _executionTime;
+    _executionTime += outerTimeStamp.elapsedMSec();
+    if (overrun) {
+        for (auto overrunItem : overrunItems) {
+            SG_LOG(SG_EVENT, SG_ALERT, "Subsystem "
+                << overrunItem.first
+                << " total "
+                << std::setw(6) << std::fixed << std::setprecision(2) << std::right << _timerStats[overrunItem.first]
+                << "s overrun "
+                << std::setw(6) << std::fixed << std::setprecision(2) << std::right << overrunItem.second
+                << "ms");
+            auto m = std::find_if(_members.begin(), _members.end(), [overrunItem](const Member* m) {
+                if (m->name == overrunItem.first)
+                    return true;
+                return false;
+            });
+            if (m != _members.end()) {
+                auto member = *m;
+                if (overrunItems[member->name]) {
+                    TimerStats sst;
+                    member->reportTimingStats(&_lastTimerStats);
+                    //if (lvTimerStats[member->name] != _timerStats[member->name]) {
+                    //    SG_LOG(SG_EVENT, SG_ALERT,
+                    //        " +" << std::setw(6) << std::left << (_timerStats[member->name] - lvTimerStats[member->name])
+                    //        << " total " << std::setw(6) << std::left << _timerStats[member->name]
+                    //        << " " << member->name
+                    //    );
+                    //}
+                }
+            }
+        }
+    }
+
+    if (reportTimingStatsRequest) {
+        reportTimingStats(nullptr);
+        //for (auto member : _members) {
+        //    member->mergeTimerStats(_timerStats);
+        //    if (_timerStats.size()) {
+        //        SG_LOG(SG_EVENT, SG_ALERT, "" << std::setw(6) << std::fixed << std::setprecision(2) << std::left << _timerStats[member->name]
+        //            << ": " << member->name);
+        //        for (auto item : _timerStats) {
+        //            if (item.second > 0)
+        //                SG_LOG(SG_EVENT, SG_ALERT, "  " << std::setw(6) << std::left << item.second << " " << item.first);
+        //        }
+        //    }
+        //}
+    }
+    _lastTimerStats.clear();
+    _lastTimerStats.insert(_timerStats.begin(), _timerStats.end());
+
+}
+void SGSubsystem::reportTimingStats(TimerStats *__lastValues) {
+    std::string _name = "";
+
+    bool reportDeltas = __lastValues != nullptr;
+    __lastValues = &_lastTimerStats;
+    std::ostringstream t;
+    if (reportDeltas) {
+        auto deltaT = _executionTime - _lastExecutionTime;
+        if (deltaT != 0) {
+            t << subsystemInstanceId() << "(+" << std::setprecision(2) << std::right << deltaT << "ms).";
+            _name = t.str();
+        }
+    }
+    else {
+        SG_LOG(SG_EVENT, SG_ALERT, "SubSystem: " << _name << " " << std::setw(6) << std::setprecision(4) << std::right << _executionTime / 1000.0 << "s");
+    }
+    for (auto item : _timerStats) {
+        std::ostringstream output;
+        if (item.second > 0) {
+            if (reportDeltas)
+            {
+                auto delta = item.second - (*__lastValues)[item.first];
+                if (delta != 0) {
+                    output 
+                        << "  +" << std::setw(6) << std::setprecision(4) << std::left << (delta * 1000.0)
+                        << _name << item.first
+                        << "  total " << std::setw(6) << std::setprecision(4) << std::right << item.second << "s " 
+                        ;
+                }
+            }
+            else
+                output << "  " << std::setw(6) << std::setprecision(4) << std::right << item.second << "s " << item.first;
+            if (output.str().size())
+                SG_LOG(SG_EVENT, SG_ALERT, output.str());
+        }
+    }
 }
 
+void SGSubsystemGroup::reportTimingStats(TimerStats *_lastValues) {
+    SGSubsystem::reportTimingStats(_lastValues);
+
+    std::string _name = subsystemInstanceId();
+    if (!_name.size())
+        _name = typeid(this).name();
+    if (_lastValues) {
+        auto deltaT = _executionTime - _lastExecutionTime;
+        if (deltaT != 0) {
+            SG_LOG(SG_EVENT, SG_ALERT, 
+                " +" << std::setw(6) << std::setprecision(4) << std::right << deltaT << "ms "
+                << subsystemInstanceId() );
+        }
+    }
+    else
+        SG_LOG(SG_EVENT, SG_ALERT, "SubSystemGroup: " << subsystemInstanceId() << " " << std::setw(6) << std::setprecision(4) << std::right << _executionTime / 1000.0 << "s");
+    for (auto member : _members) {
+        member->reportTimingStats(_lastValues);
+    }
+    _lastTimerStats.clear();
+    _lastTimerStats.insert(_timerStats.begin(), _timerStats.end());
+
+}
 void
 SGSubsystemGroup::reportTiming(void)
 {
@@ -441,11 +574,11 @@ SGSubsystemGroup::set_subsystem (const string &name, SGSubsystem * subsystem,
     if (name.empty()) {
         SG_LOG(SG_GENERAL, SG_DEV_WARN, "adding subsystem to group without a name");
         // TODO, make this an exception in the future
-    } else if (subsystem->name().empty()) {
+    } else if (subsystem->subsystemId().empty()) {
         subsystem->set_name(name);
-    } else if (name != subsystem->name()) {
+    } else if (name != subsystem->subsystemId()) {
         SG_LOG(SG_GENERAL, SG_DEV_WARN, "adding subsystem to group with name '" << name
-               << "', but name() returns '" << subsystem->name() << "'");
+               << "', but subsystemId() returns '" << subsystem->subsystemId() << "'");
     }
     
     notifyWillChange(subsystem, State::ADD);
@@ -481,7 +614,7 @@ SGSubsystemGroup::set_subsystem (const string &name, SGSubsystem * subsystem,
 void
 SGSubsystemGroup::set_subsystem (SGSubsystem * subsystem, double min_step_sec)
 {
-    set_subsystem(subsystem->name(), subsystem, min_step_sec);
+    set_subsystem(subsystem->subsystemId(), subsystem, min_step_sec);
 }
 
 SGSubsystem *
@@ -514,6 +647,14 @@ SGSubsystemGroup::remove_subsystem(const string &name)
         // found it, great
         const auto sub = (*it)->subsystem;
         
+        // ensure we clear the name during shutdown. This ensures that
+        // a lookup during shutdown fails, which is the desired behaviour.
+        // for an example see: https://sourceforge.net/p/flightgear/codetickets/2217/
+        // where we looked up Nasal during Nasal shutdown
+        // without this, the SubsystemMgr re-caches a dead pointer
+        // after doing its clear()
+        (*it)->name.clear();
+        
         if (_state != State::INVALID) {
             // transition out correctly
             if ((_state >= State::INIT) && (_state < State::SHUTDOWN)) {
@@ -609,6 +750,7 @@ auto SGSubsystemGroup::get_member(const string &name, bool create) -> Member*
 
         Member* m = new Member;
         m->name = name;
+        _timerStats[name] = 0;
         _members.push_back(m);
         return _members.back();
     }
@@ -653,6 +795,11 @@ SGSubsystemGroup::Member::Member (const Member &)
 SGSubsystemGroup::Member::~Member ()
 {
 }
+void SGSubsystemGroup::Member::mergeTimerStats(SGSubsystem::TimerStats &stats) {
+    stats.insert(subsystem->_timerStats.begin(), subsystem->_timerStats.end());
+    //for (auto ts : subsystem->_timerStats)
+    //    ts.second = 0;
+}
 
 void
 SGSubsystemGroup::Member::update (double delta_time_sec)
@@ -666,10 +813,15 @@ SGSubsystemGroup::Member::update (double delta_time_sec)
         return;
     }
     
+    SGTimeStamp oTimer;
     try {
-      subsystem->update(elapsed_sec);
-      elapsed_sec = 0;
-    } catch (sg_exception& e) {
+        oTimer.stamp();
+        subsystem->update(elapsed_sec);
+        subsystem->_lastExecutionTime = subsystem->_executionTime;
+        subsystem->_executionTime += oTimer.elapsedMSec();
+        elapsed_sec = 0;
+    }
+    catch (sg_exception& e) {
       SG_LOG(SG_GENERAL, SG_ALERT, "caught exception processing subsystem:" << name
         << "\nmessage:" << e.getMessage());
       
@@ -710,7 +862,7 @@ SGSubsystemMgr::SGSubsystemMgr () :
 #endif
     
     for (int i = 0; i < MAX_GROUPS; i++) {
-        auto g = new SGSubsystemGroup;
+        auto g = new SGSubsystemGroup();
         g->set_manager(this);
         _groups[i].reset(g);
     }
@@ -746,6 +898,7 @@ SGSubsystemMgr::init ()
             _groups[i]->init();
 }
 
+
 SGSubsystem::InitStatus
 SGSubsystemMgr::incrementalInit()
 {
@@ -802,9 +955,12 @@ SGSubsystemMgr::unbind ()
 void
 SGSubsystemMgr::update (double delta_time_sec)
 {
+    SGTimeStamp timeStamp;
+
     for (int i = 0; i < MAX_GROUPS; i++) {
         _groups[i]->update(delta_time_sec);
     }
+    reportTimingStatsRequest = false;
 }
 
 void
@@ -890,9 +1046,9 @@ SGSubsystemMgr::get_subsystem (const string &name) const
 }
 
 SGSubsystem*
-SGSubsystemMgr::get_subsystem(const std::string &name, const std::string& instanceName) const
+SGSubsystemMgr::get_subsystem(const std::string &name, const std::string& subsystemInstanceId) const
 {
-    return get_subsystem(name + SUBSYSTEM_NAME_SEPARATOR + instanceName);
+    return get_subsystem(name + SUBSYSTEM_NAME_SEPARATOR + subsystemInstanceId);
 }
 
 
@@ -1025,7 +1181,7 @@ SGSubsystemMgr::create(const std::string& name)
 }
 
 SGSubsystemRef
-SGSubsystemMgr::createInstance(const std::string& name, const std::string& instanceName)
+SGSubsystemMgr::createInstance(const std::string& name, const std::string& subsystemInstanceId)
 {
     auto it = findRegistration(name);
     auto &global_registrations = getGlobalRegistrations();
@@ -1042,7 +1198,7 @@ SGSubsystemMgr::createInstance(const std::string& name, const std::string& insta
         throw sg_exception("SGSubsystemMgr::create: functor failed to create an instsance of " + name);
     }
     
-    const auto combinedName = name + SUBSYSTEM_NAME_SEPARATOR + instanceName;
+    const auto combinedName = name + SUBSYSTEM_NAME_SEPARATOR + subsystemInstanceId;
     ref->set_name(combinedName);
     return ref;
 }
@@ -1127,7 +1283,7 @@ namespace {
         
         // allow override of the name but defaultt o the subsystem name
         std::string name = arg->getStringValue("name");
-        std::string instanceName = arg->getStringValue("instance");
+        std::string subsystemInstanceId = arg->getStringValue("instance");
 
         if (name.empty()) {
             // default name to subsystem name, but before we parse any instance name
@@ -1136,19 +1292,19 @@ namespace {
         
         auto separatorPos = subsystem.find(SUBSYSTEM_NAME_SEPARATOR);
         if (separatorPos != std::string::npos) {
-            if (!instanceName.empty()) {
+            if (!subsystemInstanceId.empty()) {
                 SG_LOG(SG_GENERAL, SG_WARN, "Specified a composite subsystem name and an instance name, please do one or the other: "
-                        << instanceName << " and " << subsystem);
+                        << subsystemInstanceId << " and " << subsystem);
                 return false;
             }
             
-            instanceName = subsystem.substr(separatorPos + 1);
+            subsystemInstanceId = subsystem.substr(separatorPos + 1);
             subsystem = subsystem.substr(0, separatorPos);
         }
         
         SGSubsystem* sub = nullptr;
-        if (!instanceName.empty()) {
-            sub = manager->createInstance(subsystem, instanceName);
+        if (!subsystemInstanceId.empty()) {
+            sub = manager->createInstance(subsystem, subsystemInstanceId);
         } else {
             sub = manager->create(subsystem);
         }
@@ -1164,7 +1320,7 @@ namespace {
         
         double minTime = arg->getDoubleValue("min-time-sec", 0.0);
         
-        const auto combinedName = subsystem + SUBSYSTEM_NAME_SEPARATOR + instanceName;
+        const auto combinedName = subsystem + SUBSYSTEM_NAME_SEPARATOR + subsystemInstanceId;
         manager->add(combinedName.c_str(),
                      sub,
                      group,
